<?xml version="1.0" encoding="UTF-8"?>
<!-- Copyright (C) 2012 Oleh Radvanskyj -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions">
	<xsl:output method="html" encoding="UTF-8" />

	<!-- inner classes -->
	
	<xsl:template match="interface" mode="interface_header">
		Interface : 
		<xsl:choose>
			<xsl:when test="@visibility = 'private'"> PRIVATE </xsl:when>
			<xsl:when test="@visibility = 'public'"> PUBLIC </xsl:when>
			<xsl:when test="@visibility = 'protected'"> PROTECTED </xsl:when>
			<xsl:when test="@visibility = 'package'"> PACKAGE </xsl:when>		
		</xsl:choose>
		<xsl:if test="@modifiers"> <xsl:value-of select="@modifiers" /> </xsl:if>
		<xsl:value-of select="@name" />
		<xsl:if test="parameterized-types">
		&lt;
			<xsl:for-each select="parameterized-types/parameterized-type">
				<xsl:value-of select="@name" /> 
				<xsl:if test="@type">
					<xsl:choose>
						<xsl:when test="@class"> extends <a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
						<xsl:otherwise> extends <xsl:value-of select="@type" /></xsl:otherwise>
					</xsl:choose>
				</xsl:if>
				<xsl:if test="not(position() = last())"> , </xsl:if>
			</xsl:for-each>
		&gt;
		</xsl:if>
		<xsl:if test="implementation">
			implements
			<xsl:for-each select="implementation">
				<xsl:choose>
					<xsl:when test="@reference"><a href="#{@reference}"><xsl:value-of select="@interface" /></a></xsl:when>
					<xsl:otherwise><xsl:value-of select="@interface" /></xsl:otherwise>
				</xsl:choose>
				<xsl:if test="not(position() = last())"> , </xsl:if>
			</xsl:for-each>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="interface" mode="inner_interface">
		<xsl:choose>
			<xsl:when test="@id"><h4 id="{@id}" onclick="toggleTag('{generate-id(.)}')"><xsl:apply-templates select="." mode="interface_header" /></h4></xsl:when>
			<xsl:otherwise><h4 id="{generate-id(.)}-header" onclick="toggleTag('{generate-id(.)}')"><xsl:apply-templates select="." mode="interface_header" /></h4></xsl:otherwise>
		</xsl:choose>		
		<div id="{generate-id(.)}" style="display : none" class="classes">
			<xsl:apply-templates select="description" />
			<xsl:apply-templates select="definition" />
			<xsl:if test="responsibility">
				<xsl:choose>
					<xsl:when test="count(responsibility) = 1"><h4>Responsibility</h4></xsl:when>
					<xsl:otherwise><h4>Responsibilities</h4></xsl:otherwise>
				</xsl:choose>
				<ul class="responsibilities">
					<xsl:apply-templates select="responsibility" />
				</ul>
			</xsl:if>
			<xsl:if test="implementation">
				<xsl:choose>
					<xsl:when test="count(implementation) = 1"><h4>Implements interface</h4></xsl:when>
					<xsl:otherwise><h4>Implements interfaces</h4></xsl:otherwise>
				</xsl:choose>
				<ul class="implementations">
					<xsl:apply-templates select="implementation" />
				</ul>
			</xsl:if>
			<xsl:apply-templates select="parameterized-types" />
			<xsl:if test="method">
				<h4>Methods</h4>
				<ul>
					<xsl:apply-templates select="method" />
				</ul>
			</xsl:if>
			<xsl:apply-templates select="diagrams" />
		</div>		
	</xsl:template>
	
	<xsl:template match="function/classes/interface | test-procedure/classes/interface | constructor/classes/interface | destructor/classes/interface | method/classes/interface">
		<xsl:apply-templates select="." mode="inner_interface" />
	</xsl:template>	

	<xsl:template match="class" mode="class_header">
		Class : 
		<xsl:choose>
			<xsl:when test="@visibility = 'private'"> PRIVATE </xsl:when>
			<xsl:when test="@visibility = 'public'"> PUBLIC </xsl:when>
			<xsl:when test="@visibility = 'protected'"> PROTECTED </xsl:when>
			<xsl:when test="@visibility = 'package'"> PACKAGE </xsl:when>		
		</xsl:choose>
		<xsl:if test="@modifiers"> <xsl:value-of select="@modifiers" /> </xsl:if>
		<xsl:value-of select="@name" />
		<xsl:if test="parameterized-types">
		&lt;
			<xsl:for-each select="parameterized-types/parameterized-type">
				<xsl:value-of select="@name" /> 
				<xsl:if test="@type">
					<xsl:choose>
						<xsl:when test="@class"> extends <a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
						<xsl:otherwise> extends <xsl:value-of select="@type" /></xsl:otherwise>
					</xsl:choose>
				</xsl:if>
				<xsl:if test="not(position() = last())"> , </xsl:if>
			</xsl:for-each>
		&gt;
		</xsl:if>
		<xsl:if test="superclass">
			extends 
			<xsl:for-each select="superclass">
				<xsl:choose>
					<xsl:when test="@class"><a href="#{@class}"><xsl:value-of select="@name" /></a></xsl:when>
					<xsl:otherwise><xsl:value-of select="@name" /></xsl:otherwise>
				</xsl:choose>
				<xsl:if test="not(position() = last())"> , </xsl:if>
			</xsl:for-each>
		</xsl:if>
		<xsl:if test="implementation">
			implements
			<xsl:for-each select="implementation">
				<xsl:choose>
					<xsl:when test="@reference"><a href="#{@reference}"><xsl:value-of select="@interface" /></a></xsl:when>
					<xsl:otherwise><xsl:value-of select="@interface" /></xsl:otherwise>
				</xsl:choose>
				<xsl:if test="not(position() = last())"> , </xsl:if>
			</xsl:for-each>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="class" mode="inner_class">
		<xsl:choose>
			<xsl:when test="@id"><h4 id="{@id}" onclick="toggleTag('{generate-id(.)}')"><xsl:apply-templates select="." mode="class_header" /></h4></xsl:when>
			<xsl:otherwise><h4 id="{generate-id(.)}-header" onclick="toggleTag('{generate-id(.)}')"><xsl:apply-templates select="." mode="class_header" /></h4></xsl:otherwise>
		</xsl:choose>		
		<div id="{generate-id(.)}" style="display:none" class="classes">
			<xsl:apply-templates select="description" />
			<xsl:apply-templates select="definition" />
			<xsl:if test="responsibility">
				<xsl:choose>
					<xsl:when test="count(responsibility) = 1"><h4>Responsibility</h4></xsl:when>
					<xsl:otherwise><h4>Responsibilities</h4></xsl:otherwise>
				</xsl:choose>
				<ul>
					<xsl:apply-templates select="responsibility" />
				</ul>
			</xsl:if>
			<xsl:if test="superclass">
				<xsl:choose>
					<xsl:when test="count(superclass) > 1"><h4>Extends classes</h4></xsl:when>
					<xsl:otherwise><h4>Extends class</h4></xsl:otherwise>
				</xsl:choose>
				<ul>
					<xsl:apply-templates select="superclass" />
				</ul>
			</xsl:if>
			<xsl:if test="implementation">
				<xsl:choose>
					<xsl:when test="count(implementation) = 1"><h4>Implements interface</h4></xsl:when>
					<xsl:otherwise><h4>Implements interfaces</h4></xsl:otherwise>
				</xsl:choose>
				<ul>
					<xsl:apply-templates select="implementation" />
				</ul>
			</xsl:if>
			<xsl:apply-templates select="parameterized-types" />
			<xsl:apply-templates select="feature" />
			<xsl:apply-templates select="attributes" />
			<xsl:apply-templates select="operations" />
			<xsl:apply-templates select="static" />
			<xsl:apply-templates select="thread" />
			<xsl:apply-templates select="diagrams" />
			<xsl:apply-templates select="test-procedures/test-procedure" />
		</div>
	</xsl:template>
	
	<xsl:template match="function/classes/class | test-procedure/classes/class | constructor/classes/class | destructor/classes/class | method/classes/class">
		<xsl:apply-templates select="." mode="inner_class" />
	</xsl:template>	
	
	<!-- content of classes -->
		
	<xsl:template match="interface/description | class/description | feature/description">
		<h4>Description</h4>
		<p class="indent"><xsl:apply-templates /></p>
	</xsl:template>
  	
	<xsl:template match="interface/definition | class/definition">
		<h4>Definition</h4>
		<p class="indent"><xsl:apply-templates /></p>
	</xsl:template>
	
	<xsl:template match="interface/responsibility | class/responsibility | feature/responsibility">
		<li><xsl:apply-templates /></li>
	</xsl:template>
	
	<xsl:template match="superclass">
		<li>
			<xsl:choose>
				<xsl:when test="@class"><a href="#{@class}"><xsl:value-of select="@name" /></a></xsl:when>
				<xsl:otherwise><xsl:value-of select="@name" /></xsl:otherwise>
			</xsl:choose>
		 </li>
	</xsl:template>
	
	<xsl:template match="interface/implementation | class/implementation">
		<li>
			<xsl:choose>
				<xsl:when test="@reference"><a href="#{@reference}"><xsl:value-of select="@interface" /></a></xsl:when>
				<xsl:otherwise><xsl:value-of select="@interface" /></xsl:otherwise>
			</xsl:choose>
	    </li>
	</xsl:template>
	
	<xsl:template match="interface/parameterized-types | class/parameterized-types">
		<xsl:choose>
			<xsl:when test="count(parameterized-type) = 1"><h4>Parameterized type</h4></xsl:when>
			<xsl:otherwise><h4>Parameterized type</h4></xsl:otherwise>
		</xsl:choose>
		<ul>
			<xsl:apply-templates select="parameterized-type" />
		</ul>
	</xsl:template>
	
	<xsl:template match="interface/parameterized-types/parameterized-type | class/parameterized-types/parameterized-type">
		<li>
			<xsl:value-of select="@name" /> 
			<xsl:if test="@type">
				<xsl:choose>
					<xsl:when test="@class"> extends <a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
					<xsl:otherwise> extends <xsl:value-of select="@type" /></xsl:otherwise>
				</xsl:choose>
			</xsl:if>
			<xsl:if test="@description">
				<em class="comment"> { <xsl:value-of select="@description" /> } </em>
			</xsl:if>
		</li>
	</xsl:template>
	
	<xsl:template match="feature">
		<h4 onclick="toggleTag('{generate-id(.)}')">Feature : <xsl:value-of select="@name" /></h4>
		<div id="{generate-id(.)}" class="feature" style="display:none">
			<xsl:apply-templates select="description" />
			<xsl:if test="responsibility">
				<xsl:choose>
					<xsl:when test="count(responsibility) = 1"><h4>Responsibility</h4></xsl:when>
					<xsl:otherwise><h4>Responsibilities</h4></xsl:otherwise>
				</xsl:choose>
				<ul>
					<xsl:apply-templates select="responsibility" />
				</ul>
			</xsl:if>
			<xsl:apply-templates select="feature" />
			<xsl:apply-templates select="attributes" />
			<xsl:apply-templates select="operations" />
			<xsl:apply-templates select="static" />
			<xsl:apply-templates select="thread" />
			<xsl:apply-templates select="diagrams" />
			<xsl:apply-templates select="test-procedures/test-procedure" />
		</div>
	</xsl:template>
	
	<xsl:template match="attributes">
		<h4>Attributes</h4>
		<ul>
			<xsl:if test="interface">
				Interfaces
				<ul class="attributes"><xsl:apply-templates select="interface" /></ul>
			</xsl:if>
			<xsl:if test="class">
				Classes
				<ul class="attributes"><xsl:apply-templates select="class" /></ul>
			</xsl:if>
			<xsl:if test="types">
				Types
				<ul class="attributes"><xsl:apply-templates select="types" /></ul>
			</xsl:if>
			<xsl:if test="constant">
				Constants
				<ul class="attributes"><xsl:apply-templates select="constant" /></ul>
			</xsl:if>
			<xsl:if test="variable">
				Variables
				<ul class="attributes"><xsl:apply-templates select="variable" /></ul>
			</xsl:if>
			<xsl:if test="element">
				Elements
				<ul class="attributes"><xsl:apply-templates select="element" /></ul>
			</xsl:if>
		</ul>
	</xsl:template>
	
	<xsl:template match="element">
		<li>
			<xsl:if test="@static or @visibility or @modifiers">
				<xsl:if test="@static = 'true'"> STATIC </xsl:if>
				<xsl:choose>
					<xsl:when test="@visibility = 'public'"> PUBLIC </xsl:when>
					<xsl:when test="@visibility = 'protected'"> PROTECTED </xsl:when>
					<xsl:when test="@visibility = 'private'"> PRIVATE </xsl:when>
					<xsl:when test="@visibility = 'pakage'"> PACKAGE </xsl:when>
				</xsl:choose>
				<xsl:value-of select="@modifiers" />
			</xsl:if>
			<xsl:value-of select="@name" />
			:
			<xsl:choose>
				<xsl:when test="@primitive">
					<xsl:choose>
						<xsl:when test="@primitive = 'integer'">INTEGER</xsl:when>
						<xsl:when test="@primitive = 'real'">REAL</xsl:when>
						<xsl:when test="@primitive = 'pointer'">POINTER</xsl:when>
						<xsl:when test="@primitive = 'character'">CHARACTER</xsl:when>
						<xsl:when test="@primitive = 'string'">STRING</xsl:when>
						<xsl:when test="@primitive = 'natural'">NATURAL</xsl:when>
						<xsl:when test="@primitive = 'boolean'">BOOLEAN</xsl:when>
						<xsl:when test="@primitive = 'variant'">VARIANT</xsl:when>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:choose>
						<xsl:when test="@class"><a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
						<xsl:otherwise><xsl:value-of select="@type" /></xsl:otherwise>
					</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:if test="@value">
				= <xsl:value-of select="@value" />
			</xsl:if>
			<xsl:if test="@purpose">
				<em class="comment"> { PURPOSE : <xsl:value-of select="@purpose" /> } </em>
			</xsl:if>
			<xsl:if test="@description">
				<em class="comment"> { <xsl:value-of select="@description" /> } </em>
			</xsl:if>
		</li>
	</xsl:template>
	
	<xsl:template match="attributes/interface">
		<li>
			<xsl:apply-templates select="." mode="inner_interface" />
		</li>
	</xsl:template>
	
	<xsl:template match="attributes/class">
		<li>
			<xsl:apply-templates select="." mode="inner_class" />
		</li>
	</xsl:template>
		
	<xsl:template match="operations">
		<h4>Operations</h4>
		<ul>
			<xsl:apply-templates select="constructor" />
			<xsl:apply-templates select="destructor" />
			<xsl:apply-templates select="method" />
			<!--
			<xsl:if test="constructor">
				<li>Constructors
					<ul><xsl:apply-templates select="constructor" /></ul>
				</li>
			</xsl:if>
			<xsl:if test="destructor">
				<li>Destructors
					<ul><xsl:apply-templates select="destructor" /></ul>
				</li>
			</xsl:if>
			<xsl:if test="method">
				<li>Methods
					<ul><xsl:apply-templates select="method" /></ul>
				</li>
			</xsl:if>
-->
		</ul>
	</xsl:template>

	<xsl:template match="class/static | feature/static">
		<h4 onclick="toggleAlgorithm('{generate-id(.)}')">Static</h4>
		<ol id="{generate-id(.)}" class="algorithm" state="collapsed"><xsl:apply-templates /></ol>	
	</xsl:template>
	
	<xsl:template match="class/thread | feature/thread">
		<h4 onclick="toggleAlgorithm('{generate-id(.)}')">Thread</h4>
		<ol id="{generate-id(.)}" class="algorithm" state="collapsed"><xsl:apply-templates /></ol>	
	</xsl:template>
	
</xsl:stylesheet>
