<?xml version="1.0" encoding="UTF-8"?>
<!-- Copyright (C) 2012 Oleh Radvanskyj -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions">
	<xsl:output method="html" encoding="UTF-8" />

	<!-- classes -->
	
	<xsl:template match="model | package" mode="table_of_contents_interfaces">
		<xsl:if test="classes/interface or use-case/classes/interface">
			<li>
				<xsl:choose>
					<xsl:when test="classes"><a href="#{generate-id(classes)}">Interfaces</a></xsl:when>
					<xsl:otherwise>Interfaces</xsl:otherwise>
				</xsl:choose>
				<ul>
					<xsl:for-each select="classes/interface">
						<li>
							<xsl:choose>
								<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
								<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
							</xsl:choose>
						</li>
					</xsl:for-each>
					<xsl:for-each select="use-case/classes/interface">
						<li>
							<xsl:choose>
								<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
								<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
							</xsl:choose>
						</li>
					</xsl:for-each>
				</ul>
			</li>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="model | package" mode="table_of_contents_classes">
		<xsl:if test="classes/class or use-case/classes/class">
			<li>
				<xsl:choose>
					<xsl:when test="classes"><a href="#{generate-id(classes)}">Classes</a></xsl:when>
					<xsl:otherwise>Classes</xsl:otherwise>
				</xsl:choose>
				<ul>
					<xsl:for-each select="classes/class">
						<li>
							<xsl:choose>
								<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
								<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
							</xsl:choose>
						</li>
					</xsl:for-each>
					<xsl:for-each select="use-case/classes/class">
						<li>
							<xsl:choose>
								<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
								<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
							</xsl:choose>
						</li>
					</xsl:for-each>
				</ul>
			</li>
		</xsl:if>
	</xsl:template>
		
	<xsl:template match="model/classes">
		<h2 id="{generate-id(.)}">Classes</h2>
		<xsl:apply-templates />
	</xsl:template>
		
	<xsl:template match="package/classes | segment/classes">
		<h4 id="{generate-id(.)}">Classes</h4>
		<div class="level_of_model">
			<xsl:apply-templates />
		</div>
	</xsl:template>
	
	<xsl:template match="model/use-case/classes">
		<xsl:apply-templates />
	</xsl:template>
		
	<xsl:template match="model/classes/interface | model/use-case/classes/interface">
		<xsl:choose>
			<xsl:when test="@id"><h3 id="{@id}" onclick="toggleTag('{generate-id(.)}')">Interface : <xsl:value-of select="@name" /></h3></xsl:when>
			<xsl:otherwise><h3 id="{generate-id(.)}-header" onclick="toggleTag('{generate-id(.)}')">Interface : <xsl:value-of select="@name" /></h3></xsl:otherwise>
		</xsl:choose>		
		<div id="{generate-id(.)}" style="display : none" class="classes">
			<xsl:apply-templates select="description" />
			<xsl:apply-templates select="definition" />
			<xsl:if test="responsibility">
				<xsl:choose>
					<xsl:when test="count(responsibility) = 1"><h4>Responsibility</h4></xsl:when>
					<xsl:otherwise><h4>Responsibilities</h4></xsl:otherwise>
				</xsl:choose>
				<ul class="responsibilities">
					<xsl:apply-templates select="responsibility" />
				</ul>
			</xsl:if>
			<xsl:if test="implementation">
				<xsl:choose>
					<xsl:when test="count(implementation) = 1"><h4>Implements interface</h4></xsl:when>
					<xsl:otherwise><h4>Implements interfaces</h4></xsl:otherwise>
				</xsl:choose>
				<ul class="implementations">
					<xsl:apply-templates select="implementation" />
				</ul>
			</xsl:if>
			<xsl:apply-templates select="parameterized-types" />
			<xsl:if test="method">
				<h4>Methods</h4>
				<ul>
					<xsl:apply-templates select="method" />
				</ul>
			</xsl:if>
			<xsl:apply-templates select="diagrams" />
		</div>
	</xsl:template>

	<xsl:template match="model/classes/class | model/use-case/classes/class">
		<xsl:choose>
			<xsl:when test="@id"><h3 id="{@id}" onclick="toggleTag('{generate-id(.)}')">Class : <xsl:value-of select="@name" /></h3></xsl:when>
			<xsl:otherwise><h3 id="{generate-id(.)}-header" onclick="toggleTag('{generate-id(.)}')">Class : <xsl:value-of select="@name" /></h3></xsl:otherwise>
		</xsl:choose>		
		<div id="{generate-id(.)}" style="display : none" class="classes">
			<xsl:apply-templates select="description" />
			<xsl:apply-templates select="definition" />
			<xsl:if test="responsibility">
				<xsl:choose>
					<xsl:when test="count(responsibility) = 1"><h4>Responsibility</h4></xsl:when>
					<xsl:otherwise><h4>Responsibilities</h4></xsl:otherwise>
				</xsl:choose>
				<ul>
					<xsl:apply-templates select="responsibility" />
				</ul>
			</xsl:if>
			<xsl:if test="superclass">
				<xsl:choose>
					<xsl:when test="count(superclass) > 1"><h4>Extends classes</h4></xsl:when>
					<xsl:otherwise><h4>Extends class</h4></xsl:otherwise>
				</xsl:choose>
				<ul>
					<xsl:apply-templates select="superclass" />
				</ul>
			</xsl:if>
			<xsl:if test="implementation">
				<xsl:choose>
					<xsl:when test="count(implementation) = 1"><h4>Implements interface</h4></xsl:when>
					<xsl:otherwise><h4>Implements interfaces</h4></xsl:otherwise>
				</xsl:choose>
				<ul>
					<xsl:apply-templates select="implementation" />
				</ul>
			</xsl:if>
			<xsl:apply-templates select="parameterized-types" />
			<xsl:apply-templates select="feature" />
			<xsl:apply-templates select="attributes" />
			<xsl:apply-templates select="operations" />
			<xsl:apply-templates select="static" />
			<xsl:apply-templates select="thread" />
			<xsl:apply-templates select="diagrams" />
			<xsl:apply-templates select="test-procedures/test-procedure" />
		</div>
	</xsl:template>
	
	<xsl:template match="function/classes | test-procedure/classes | constructor/classes | destructor/classes | method/classes">
		<xsl:apply-templates />
	</xsl:template>
	
	<xsl:template match="package/classes/interface | package/use-case/classes/interface | segment/classes/interface">
		<xsl:apply-templates select="." mode="inner_interface" />
	</xsl:template>
		
	<xsl:template match="package/classes/class | package/use-case/classes/class | segment/classes/class">
		<xsl:apply-templates select="." mode="inner_class" />
	</xsl:template>
		
</xsl:stylesheet>
