<?xml version="1.0" encoding="UTF-8"?>
<!-- Copyright (C) 2012 Oleh Radvanskyj -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions">
  <xsl:output method="html" encoding="UTF-8" indent="yes"/>
  <xsl:include href="common/htmlSubset.xslt" />
  
  <xsl:template match="proposal">
<html>
  <head>
    <title>Proposal</title>
    <link rel="stylesheet" type="text/css" href=".blueprint/css/proposal.css" /> 
    <xsl:for-each select="stylesheet-link">
		<link rel="stylesheet" type="text/css" href="{@href}" />
    </xsl:for-each>
  </head>
  <body>
    <h1>Proposal</h1>
    <xsl:apply-templates  select="problem" />
    <xsl:apply-templates select="vision" />
    <xsl:apply-templates select="functional-requirements" />
    <xsl:apply-templates select="non-functional-requirements" />
    <xsl:apply-templates select="glossary" />
  </body>
</html>
  </xsl:template>
  
  <xsl:template match="problem">
    <h2>Problem</h2>
    <p><xsl:apply-templates /></p>
  </xsl:template>

  <xsl:template match="vision">
    <h2>Vision</h2>
    <p><xsl:apply-templates /></p>
  </xsl:template>
  
  <xsl:template match="functional-requirements">
    <h2>Functional-requirements</h2>
    <ol>
      <xsl:apply-templates select="package" />
      <xsl:apply-templates select="requirement" />
    </ol>
  </xsl:template>

  <xsl:template match="non-functional-requirements">
    <h2>Non-functional requirements</h2>
    <ol>
      <xsl:apply-templates select="package" />
      <xsl:apply-templates select="requirement" />
    </ol>
  </xsl:template>
  
  <xsl:template match="package">
    <li>
      <span><xsl:value-of select="@name" /></span>
      <ol>
        <xsl:apply-templates select="package" />
        <xsl:apply-templates select="requirement" />
      </ol>
    </li>
  </xsl:template>
  
  <xsl:template match="requirement">
	 <xsl:variable name="title">
		<xsl:if test="@status">Status : <xsl:value-of select="@status" />; </xsl:if>
		<xsl:if test="@benefit">Benefit : <xsl:value-of  select="@benefit" />; </xsl:if>
		<xsl:if test="@effort">Effort : <xsl:value-of select="@effort" />; </xsl:if>
		<xsl:if test="@risk">Risk : <xsl:value-of select="@risk" />; </xsl:if>
		<xsl:if test="@stability">Stability : <xsl:value-of select="@stability" />; </xsl:if>
		<xsl:if test="@target-release">Target release : <xsl:value-of select="@target-release" />; </xsl:if>
	 </xsl:variable>
    <li title="{$title}">
		<xsl:value-of select="." />
		<xsl:choose>
		  <xsl:when test="@priority = 'mast_have'">
			<span class="priority">(mast have)</span>
		  </xsl:when>
		  <xsl:when test="@priority = 'should_have'">
			<span class="priority">(should have)</span>
		  </xsl:when>
		  <xsl:when test="@priority = 'could_have'">
			<span class="priority">(could have)</span>
		  </xsl:when>
		  <xsl:when test="@priority = 'want_to_have'">
			<span class="priority">(want to have)</span>
		  </xsl:when>
		</xsl:choose>
    </li>
  </xsl:template>
  
  <xsl:template match="glossary">
    <h2>Glossary</h2>
    <table cellspacing="0" border="1" class="glossary">
      <tr>
        <th>Word</th>
        <th>Description</th>
        <xsl:if test="term/synonyms">
			<th>Synonyms</th>
        </xsl:if>
        <xsl:if test="term/homonyms">
			<th>Homonyms</th>
        </xsl:if>
      </tr>
      <xsl:apply-templates select="term" />
    </table>
  </xsl:template>
  
  <xsl:template match="term">
    <tr>
      <td><xsl:value-of select="word"/></td>
      <td><xsl:value-of select="description"/></td>
      <xsl:if test="synonyms">
		<td><ul class="synonyms">
			<xsl:for-each select="synonyms/synonym">
				<li><xsl:value-of select="." /></li>
			</xsl:for-each>
		</ul></td>
      </xsl:if>
      <xsl:if test="homonyms">
		  <td><ul class="homonyms">
			  <xsl:for-each select="homonyms/homonym">
				  <li><xsl:value-of select="." /></li>
			  </xsl:for-each>
		  </ul></td>
      </xsl:if>
    </tr>
  </xsl:template>

</xsl:stylesheet>
