<?xml version="1.0" encoding="UTF-8"?>
<!-- Copyright (C) 2012 Oleh Radvanskyj -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions">
	<xsl:output method="html" encoding="UTF-8" />
	
  <!-- use-case -->
    
	<xsl:template match="model | package" mode="table_of_contents_use_case">
		<xsl:if test="use-case">
			<li>Use cases
				<ul>
					<xsl:for-each select="use-case">
						<li>
							<a href="#{generate-id(.)}"><xsl:value-of select="@name" /></a>
							<xsl:if test="gui">
								<ul>
									<xsl:for-each select="gui/screen">
										<li>
											<xsl:choose>
												<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
												<xsl:otherwise><a href="#{generate-id(.)}"><xsl:value-of select="@name" /></a></xsl:otherwise>
											</xsl:choose>
										</li>
									</xsl:for-each>
								</ul>
							</xsl:if>
							<xsl:if test="diagrams">
								<ul>
									<xsl:for-each select="diagrams/diagram">
										<li><a href="#{generate-id(.)}"><xsl:value-of select="@name" /></a></li>
									</xsl:for-each>
								</ul>
							</xsl:if>
						</li>	
					</xsl:for-each>
				</ul>
			</li>
		</xsl:if>
	</xsl:template>
    
  <xsl:template match="model/use-case">
		<h2 id="{generate-id(.)}">
			<xsl:choose>
				<xsl:when test="@type='abstract'">Abstract use case : </xsl:when>
				<xsl:when test="@type='concrete'">Concrete use case : </xsl:when>
				<xsl:otherwise>Use case : </xsl:otherwise>
			</xsl:choose>
			<span class="usecase-id"><xsl:value-of select="@name" /></span>
			<xsl:if test="@extends">
				extends <xsl:value-of select="@extends" />
			</xsl:if>
		</h2>
		<xsl:apply-templates select="description" />
		<xsl:apply-templates select="goal" />
		<xsl:apply-templates select="concretization" />
		<xsl:apply-templates select="actors" />
		<xsl:apply-templates select="preconditions" />
		<xsl:apply-templates select="narrative" />
		<xsl:apply-templates select="postconditions" />
		<xsl:apply-templates select="alternative" />
		<xsl:apply-templates select="segment" />
		<xsl:apply-templates select="packages/package" />
		<xsl:apply-templates select="classes" />
		<xsl:apply-templates select="functions/function" />
		<xsl:apply-templates select="thread" />
		<xsl:apply-templates select="test-procedures/test-procedure" />
		<xsl:apply-templates select="test-thread" />
		<xsl:apply-templates select="artifacts" />
		<xsl:apply-templates select="gui" />
		<xsl:apply-templates select="test-scenarios" />
		<xsl:apply-templates select="diagrams" />
  </xsl:template>
  
  <xsl:template match="model/use-case/description">
    <h3>Description</h3>
    <p><xsl:apply-templates /></p>
  </xsl:template>
  
  <xsl:template match="model/use-case/goal">
    <h3>Goal</h3>
    <p><xsl:apply-templates /></p>
  </xsl:template>
  
  <xsl:template match="model/use-case/preconditions">
	<h3>Preconditions</h3>
	<xsl:choose>
		<xsl:when test="condition">
			<div class="indent">
				<ul class="conditions"><xsl:apply-templates /></ul>
			</div>  
		</xsl:when>
		<xsl:otherwise>
			<p><xsl:apply-templates /></p>
		</xsl:otherwise>
	</xsl:choose>
  </xsl:template>
  
  <xsl:template match="model/use-case/narrative">
    <h3 onclick="toggleAlgorithm('{generate-id(.)}')">Narrative</h3>
	<ol id="{generate-id(.)}" state="collapsed" class="algorithm"><xsl:apply-templates /></ol>
  </xsl:template>
  
  <xsl:template match="model/use-case/postconditions">
    <h3>Postconditions</h3>
	<xsl:choose>
		<xsl:when test="condition">
			<div class="indent">
				<ul class="conditions"><xsl:apply-templates /></ul>
			</div>  
		</xsl:when>
		<xsl:otherwise>
			<p><xsl:apply-templates /></p>
		</xsl:otherwise>
	</xsl:choose>
  </xsl:template>

  <xsl:template match="model/use-case/thread">
	<xsl:choose>
		<xsl:when test="@id">
			<h3 id="{@id}" onclick="toggleAlgorithm('{generate-id(.)}')">Thread
				<xsl:if test="@name">
					: <xsl:value-of select="@name" />
				</xsl:if>
				<xsl:if test="@description">
					( <xsl:value-of select="@description" /> )
				</xsl:if>
			</h3>
		</xsl:when>
		<xsl:otherwise>
			<h3 onclick="toggleAlgorithm('{generate-id(.)}')">Thread
				<xsl:if test="@name">
					: <xsl:value-of select="@name" />
				</xsl:if>
				<xsl:if test="@description">
					( <xsl:value-of select="@description" /> )
				</xsl:if>
			</h3>
		</xsl:otherwise>
	</xsl:choose>
    <ol id="{generate-id(.)}" state="collapsed" class="algorithm"><xsl:apply-templates /></ol>
  </xsl:template>

  <xsl:template match="model/use-case/test-thread">
	<h3 onclick="toggleAlgorithm('{generate-id(.)}')">Test thread</h3>
    <ol id="{generate-id(.)}" state="collapsed" class="algorithm"><xsl:apply-templates /></ol>    
  </xsl:template>

  <xsl:template match="model/use-case/artifacts">
    <h3>Artifacts</h3>
    <xsl:choose>
		<xsl:when test="artifact">
			<ul class="artifacts"><xsl:apply-templates /></ul>
		</xsl:when>
		<xsl:otherwise>
			<pre><xsl:apply-templates /></pre>
		</xsl:otherwise>
	</xsl:choose>
  </xsl:template>
  
  <xsl:template match="artifact">
	  <li><xsl:value-of select="@name" /> - <xsl:value-of select="@description" /></li>
  </xsl:template>

</xsl:stylesheet>
