<?xml version="1.0" encoding="UTF-8"?>
<!-- Copyright (C) 2012 Oleh Radvanskyj -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions">
	<xsl:output method="html" encoding="UTF-8" indent="yes"/>
	<xsl:include href="analysis/classes.xslt" />
	<xsl:include href="analysis/overview.xslt" />
    <xsl:include href="common/classes.xslt" />
    <xsl:include href="common/functions.xslt" />
    <xsl:include href="common/diagrams.xslt" />
    <xsl:include href="common/types.xslt" />
    <xsl:include href="common/algorithm.xslt" />
    <xsl:include href="research/researches.xslt" />

    <xsl:param name="out" as="xs:string" select="'memory'" />
	
	<xsl:template match="analysis">
		<html>
			<head>
				<title>Analysis</title>
				<link rel="stylesheet" type="text/css" href=".blueprint/css/common/table_of_contents.css" />
				<link rel="stylesheet" type="text/css" href=".blueprint/css/common/anchors.css" />
				<link rel="stylesheet" type="text/css" href=".blueprint/css/common/aspects.css" />
				<link rel="stylesheet" type="text/css" href=".blueprint/css/analysis/overview.css" />
				<link rel="stylesheet" type="text/css" href=".blueprint/css/common/researches.css" />
				<link rel="stylesheet" type="text/css" href=".blueprint/css/analysis.css" />
				<xsl:for-each select="stylesheet-link">
					<link rel="stylesheet" type="text/css" href="{@href}" />
				</xsl:for-each>
			    <script type="text/javascript" src=".blueprint/js/tag_behaviour.js"></script>
			    <script type="text/javascript" src=".blueprint/js/research.js"></script>
			</head>
			<body>
				<h1>Analysis</h1>
				<xsl:apply-templates select="." mode="table_of_contents_analysis" />
				<xsl:apply-templates select="use-case-model" />
				<xsl:apply-templates select="researches" />
				<xsl:apply-templates select="database" />
				<xsl:apply-templates select="architecture" />
				<xsl:apply-templates select="gui" />
			</body>
		</html>
	</xsl:template>
	
	<xsl:template match="analysis" mode="table_of_contents_analysis">
		<h3>Table of contents</h3>
		<ul class="contents">
			<xsl:if test="use-case-model">
				<li>
					<a href="#use_case_model">Use case model</a>
					<xsl:if test="use-case-model/diagram">
						<ul>
							<xsl:for-each select="use-case-model/diagram">
								<li><a href="#{generate-id(.)}"><xsl:value-of select="@name" /></a></li>
							</xsl:for-each>
						</ul>
					</xsl:if>
				</li>
			</xsl:if>
			<xsl:if test="researches">
				<li><a href="#{generate-id(researches)}">Researches</a></li>
			</xsl:if>
			<xsl:if test="database">
				<li>
					<a href="#database">Database</a>
					<xsl:if test="database/diagram">
						<ul>
							<xsl:for-each select="database/diagram">
								<li><a href="#{generate-id(.)}"><xsl:value-of select="@name" /></a></li>
							</xsl:for-each>
						</ul>
					</xsl:if>
				</li>
			</xsl:if>
			<xsl:if test="architecture">
				<li>
					<a href="#architecture">Architecture</a>
					<ul>
						<xsl:if test="architecture/package">
							<li>Packages
								<ul>
									<xsl:for-each select="architecture/package">
										<li>
											<xsl:choose>
												<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
												<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
											</xsl:choose>
										</li>
									</xsl:for-each>
								</ul>
							</li>
						</xsl:if>
						<xsl:if test="architecture/pd-package">
							<li>PD
								<ul>
									<xsl:for-each select="architecture/pd-package">
										<li>
											<xsl:choose>
												<xsl:when test="@name"><a href="#{generate-id(.)}"><xsl:value-of select="@name" /></a></xsl:when>
												<xsl:otherwise><a href="#{generate-id(.)}">nameless</a></xsl:otherwise>
											</xsl:choose>
										</li>
									</xsl:for-each>
								</ul>
							</li>
						</xsl:if>
						<xsl:if test="architecture/cd-package">
							<li>CD
								<ul>
									<xsl:for-each select="architecture/cd-package">
										<li>
											<xsl:choose>
												<xsl:when test="@name"><a href="#{generate-id(.)}"><xsl:value-of select="@name" /></a></xsl:when>
												<xsl:otherwise><a href="#{generate-id(.)}">nameless</a></xsl:otherwise>
											</xsl:choose>
										</li>
									</xsl:for-each>
								</ul>
							</li>
						</xsl:if>
						<xsl:if test="architecture/crc-package">
							<li>CRC
								<ul>
									<xsl:for-each select="architecture/crc-package">
										<li>
											<xsl:choose>
												<xsl:when test="@name"><a href="#{generate-id(.)}"><xsl:value-of select="@name" /></a></xsl:when>
												<xsl:otherwise><a href="#{generate-id(.)}">nameless</a></xsl:otherwise>
											</xsl:choose>
										</li>
									</xsl:for-each>
								</ul>
							</li>
						</xsl:if>
						<xsl:if test="architecture/interface">
							<li>Interfaces
								<ul>
									<xsl:for-each select="architecture/interface">
										<li>
											<xsl:choose>
												<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
												<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
											</xsl:choose>															
										</li>
									</xsl:for-each>
								</ul>
							</li>
						</xsl:if>
						<xsl:if test="architecture/class">
							<li>Classes
								<ul>
									<xsl:for-each select="architecture/class">
										<li>
											<xsl:choose>
												<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
												<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
											</xsl:choose>															
										</li>
									</xsl:for-each>
								</ul>
							</li>
						</xsl:if>
						<xsl:if test="architecture/diagram">
							<li>Diagrams
								<ul>
									<xsl:for-each select="architecture/diagram">
										<li><a href="#{generate-id(.)}"><xsl:value-of select="@name" /></a></li>
									</xsl:for-each>
								</ul>
							</li>
						</xsl:if>
					</ul>
				</li>
			</xsl:if>
			<xsl:if test="gui">
				<li>
					<a href="#gui">GUI</a>
					<xsl:if test="gui/screen">
						<ul>
							<xsl:for-each select="gui/screen">
								<li><a href="#{generate-id(.)}"><xsl:value-of select="@name" /></a></li>
							</xsl:for-each>
						</ul>
					</xsl:if>
				</li>
			</xsl:if>
		</ul>
	</xsl:template>

	<!-- use case model -->

	<xsl:template match="use-case-model">
		<h2 id="use_case_model">Use case model</h2>
		<xsl:apply-templates select="diagram" />
	</xsl:template>
	
	<xsl:template match="use-case-model/diagram | architecture/diagram">
		<xsl:choose>
			<xsl:when test="@id"><h3 id="{@id}"><xsl:value-of select="@name" /></h3></xsl:when>
			<xsl:otherwise><h3 id="{generate-id(.)}"><xsl:value-of select="@name" /></h3></xsl:otherwise>
		</xsl:choose>
		<img alt="{@name}" src="{@image}" />
	</xsl:template>

	<!-- database -->
	
	<xsl:template match="database">
		<h2 id="database">Database</h2>
		<xsl:if test="table-def">
			<h3>Tables</h3>
			<ul class="tables">
				<xsl:apply-templates select="table-def" />
			</ul>
		</xsl:if>
		<xsl:apply-templates select="diagram" />
	</xsl:template>
	
	<xsl:template match="table-def">
		<li><xsl:value-of select="." /></li>
	</xsl:template>

	<!-- architecture -->

	<xsl:template match="architecture">
		<h2 id="architecture">Architecture</h2>
		<xsl:apply-templates select="pd-package" />
		<xsl:apply-templates select="cd-package" />
		<xsl:apply-templates select="crc-package" />
		<xsl:apply-templates select="package" />
		<xsl:apply-templates select="interface" />
		<xsl:apply-templates select="class" />
		<xsl:apply-templates select="diagram" />
	</xsl:template>
	
	<xsl:template match="package" mode="table_of_contents_inner_package">
		<h4>Table of contents</h4>
		<ul class="contents">
			<xsl:if test="interface or class">
				<li><a href="#{generate-id(.)}-overview-header">Overview</a></li>
			</xsl:if>
			<xsl:if test="package">
				<li>Packages
					<ul>
						<xsl:for-each select="package">
							<li>
								<xsl:choose>
									<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
									<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
								</xsl:choose>
							</li>
						</xsl:for-each>
					</ul>
				</li>
			</xsl:if>
			<xsl:if test="pd-package">
				<li>PD
					<ul>
						<xsl:for-each select="pd-package">
							<li>
								<xsl:choose>
									<xsl:when test="@name"><a href="#{generate-id(.)}"><xsl:value-of select="@name" /></a></xsl:when>
									<xsl:otherwise><a href="#{generate-id(.)}">nameless</a></xsl:otherwise>
								</xsl:choose>
							</li>
						</xsl:for-each>
					</ul>
				</li>
			</xsl:if>
			<xsl:if test="cd-package">
				<li>CD
					<ul>
						<xsl:for-each select="cd-package">
							<li>
								<xsl:choose>
									<xsl:when test="@name"><a href="#{generate-id(.)}"><xsl:value-of select="@name" /></a></xsl:when>
									<xsl:otherwise><a href="#{generate-id(.)}">nameless</a></xsl:otherwise>
								</xsl:choose>
							</li>
						</xsl:for-each>
					</ul>
				</li>
			</xsl:if>
			<xsl:if test="crc-package">
				<li>CRC
					<ul>
						<xsl:for-each select="crc-package">
							<li>
								<xsl:choose>
									<xsl:when test="@name"><a href="#{generate-id(.)}"><xsl:value-of select="@name" /></a></xsl:when>
									<xsl:otherwise><a href="#{generate-id(.)}">nameless</a></xsl:otherwise>
								</xsl:choose>
							</li>
						</xsl:for-each>
					</ul>
				</li>
			</xsl:if>
			<xsl:if test="interface">
				<li>Interfaces
					<ul>
						<xsl:for-each select="interface">
							<li>
								<xsl:choose>
									<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
									<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
								</xsl:choose>															
							</li>
						</xsl:for-each>
					</ul>
				</li>
			</xsl:if>
			<xsl:if test="class">
				<li>Classes
					<ul>
						<xsl:for-each select="class">
							<li>
								<xsl:choose>
									<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
									<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
								</xsl:choose>															
							</li>
						</xsl:for-each>
					</ul>
				</li>
			</xsl:if>
			<xsl:if test="diagram">
				<li>Diagrams
					<ul>
						<xsl:for-each select="diagram">
							<li><a href="#{generate-id(.)}"><xsl:value-of select="@name" /></a></li>
						</xsl:for-each>
					</ul>
				</li>
			</xsl:if>
		</ul>
	</xsl:template>
	
	<xsl:template match="architecture/package">
		<xsl:choose>
			<xsl:when test="@id"><h3 id="{@id}" onclick="toggleTag('{generate-id(.)}')">Package : <xsl:value-of select="@name" /></h3></xsl:when>
			<xsl:otherwise><h3 id="{generate-id(.)}-header" onclick="toggleTag('{generate-id(.)}')">Package : <xsl:value-of select="@name" /></h3></xsl:otherwise>
		</xsl:choose>
		<div id="{generate-id(.)}" style="display:none" class="indent">
			<xsl:apply-templates select="description" />
			<xsl:apply-templates select="." mode="table_of_contents_inner_package" />
			<xsl:apply-templates select="." mode="overview" />
			<xsl:apply-templates select="pd-package" />
			<xsl:apply-templates select="cd-package" />
			<xsl:apply-templates select="crc-package" />
			<xsl:apply-templates select="package" />
			<xsl:apply-templates select="interface" />
			<xsl:apply-templates select="class" />
			<xsl:apply-templates select="diagram" />
		</div>
	</xsl:template>

	<xsl:template match="package/package">
		<xsl:choose>
			<xsl:when test="@id"><h4 id="{@id}" onclick="toggleTag('{generate-id(.)}')">Package : <xsl:value-of select="@name" /></h4></xsl:when>
			<xsl:otherwise><h4 id="{generate-id(.)}-header" onclick="toggleTag('{generate-id(.)}')">Package : <xsl:value-of select="@name" /></h4></xsl:otherwise>
		</xsl:choose>
		<div id="{generate-id(.)}" style="display:none" class="indent">
			<xsl:apply-templates select="description" />
			<xsl:apply-templates select="." mode="table_of_contents_inner_package" />
			<xsl:apply-templates select="." mode="overview" />
			<xsl:apply-templates select="cd-package" />
			<xsl:apply-templates select="crc-package" />
			<xsl:apply-templates select="package" />
			<xsl:apply-templates select="interface" />
			<xsl:apply-templates select="class" />
			<xsl:apply-templates select="diagram" />
		</div>
	</xsl:template>

	<xsl:template match="package/description">
		<h4>Description</h4>
		<div class="indent">
			<xsl:apply-templates />
		</div>
	</xsl:template>
	
	<xsl:template match="package" mode="overview">
		<xsl:if test="interface or class or package">
			<h4 id="{generate-id(.)}-overview-header" onclick="toggleTag('{generate-id(.)}-overview')">Overview</h4>
			<div id="{generate-id(.)}-overview" class="indent" style="display:none">
				<xsl:apply-templates select="." mode="pd-overview" />
				<xsl:apply-templates select="." mode="cd-overview" />
				<xsl:apply-templates select="." mode="crc-overview" />
			</div>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="architecture/package/diagram | package/package/diagram">
		<xsl:choose>
			<xsl:when test="@id"><h4 id="{@id}">Diagram : <xsl:value-of select="@name" /></h4></xsl:when>
			<xsl:otherwise><h4 id="{generate-id(.)}">Diagram : <xsl:value-of select="@name" /></h4></xsl:otherwise>
		</xsl:choose>
		<img alt="{@name}" src="{@image}" />
	</xsl:template>

	<!-- GUI -->
	
	<xsl:template match="gui">
		<h2 id="gui">GUI</h2>
		<xsl:if test="menu">
			<h3>Menus</h3>
			<ul class="menu">
				<xsl:apply-templates select="menu" />
			</ul>
		</xsl:if>
		<xsl:if test="toolbar">
			<h3>Toolbars</h3>
			<ul class="toolbar">
				<xsl:apply-templates select="toolbar" />
			</ul>
		</xsl:if>
		<xsl:apply-templates select="screen" />
	</xsl:template>
	
	<xsl:template match="menu">
		<li><xsl:value-of select="." /></li>
	</xsl:template>
		
	<xsl:template match="toolbar">
		<li><xsl:value-of select="." /></li>
	</xsl:template>
		
	<xsl:template match="screen">
		<h3 id="{generate-id(.)}"><xsl:value-of select="@name" /></h3>
		<div class="screen">
		  <img alt="{@name}" src="{@image}" style="float:left; padding-right:10px" />
		  <div style="display:inline">
			  <xsl:apply-templates />
		  </div>
		  <br style="clear:left" />
		</div>
	</xsl:template>
	
</xsl:stylesheet>
