<?xml version="1.0" encoding="UTF-8"?>
<!-- Copyright (C) 2012 Oleh Radvanskyj -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions">
	<xsl:output method="html" encoding="UTF-8" />
	
	<!-- algorithm -->
    
	<!-- if -->
	<xsl:template match="if">
		<xsl:choose>
			<xsl:when test="@function">
				<li>IF <a href="#{@function}"><xsl:value-of select="@condition" /></a></li>
			</xsl:when>
			<xsl:otherwise>
				<li>IF <xsl:value-of select="@condition" /></li>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:apply-templates select="then" />
		<xsl:apply-templates select="else-if" />
		<xsl:apply-templates select="else" />
		<li>END-IF</li>
	</xsl:template>
	<xsl:template match="then">
		<li>THEN
			<ol><xsl:apply-templates /></ol>
		</li>
	</xsl:template>
	<xsl:template match="else-if">
		<xsl:choose>
			<xsl:when test="@function">
				<li>ELSE IF <a href="#{@function}"><xsl:value-of select="@condition" /></a></li>
			</xsl:when>
			<xsl:otherwise>
				<li>ELSE IF <xsl:value-of select="@condition" /></li>
			</xsl:otherwise>
		</xsl:choose>
		<li>THEN
			<ol><xsl:apply-templates /></ol>
		</li>
	</xsl:template>
	<xsl:template match="else">
		<li>ELSE
			<ol><xsl:apply-templates /></ol>
		</li>
	</xsl:template>
	
	<!-- step -->
	<xsl:template match="step">
		<li>
			<span id="{generate-id(.)}-action"><a href="javascript:expandStep('{generate-id(.)}')"><xsl:apply-templates select="action" /></a></span> 
			<ol id="{generate-id(.)}-detailing" style="display:none">
				<xsl:apply-templates select="detailing" />
			</ol>
		</li>
	</xsl:template>
	<xsl:template match="action">
		<xsl:apply-templates />
	</xsl:template>
	<xsl:template match="detailing">
		<xsl:apply-templates />
	</xsl:template>
	
	<!-- perform -->
	<xsl:template match="perform">
		<li>
			<span id="{generate-id(.)}-action"><a href="javascript:expandStep('{generate-id(.)}')">PERFORM
				<xsl:choose>
					<xsl:when test="@function"><a href="#{@function}"><xsl:value-of select="@action" /></a></xsl:when>
					<xsl:when test="@step"><a href="#{@step}"><xsl:value-of select="@action" /></a></xsl:when>
					<xsl:otherwise><xsl:value-of select="@action" /></xsl:otherwise>
				</xsl:choose>
			</a></span> 
			<ol id="{generate-id(.)}-detailing" style="display:none">
				<xsl:apply-templates />
			</ol>
		</li>
	</xsl:template>	
	
	<!-- doing -->
	<xsl:template match="doing">
		<li><xsl:apply-templates /></li>
	</xsl:template>
	
	<!-- while -->
	<xsl:template match="while">
		<li>
			<xsl:choose>
				<xsl:when test="@function">
					WHILE <a href="#{@function}"><xsl:value-of select="@condition" /></a>
				</xsl:when>
				<xsl:otherwise>
					WHILE <xsl:value-of select="@condition" />
				</xsl:otherwise>
			</xsl:choose>
			<ol><xsl:apply-templates /></ol>
		</li>
		<li>END-WHILE</li>	
	</xsl:template>
	
	<!-- repeat -->
	<xsl:template match="repeat">
		<li>REPEAT
			<ol><xsl:apply-templates /></ol>
		</li>
		<xsl:choose>
			<xsl:when test="@function">
				<li>UNTIL <a href="#{@function}"><xsl:value-of select="@until" /></a></li>
			</xsl:when>
			<xsl:otherwise>
				<li>UNTIL <xsl:value-of select="@until" /></li>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- for -->
	<xsl:template match="for">
		<li>FOR <xsl:value-of select="@counter" /> COUNTER FROM 
			<xsl:choose>
				<xsl:when test="@from-function"><a href="#{@from-function}"><xsl:value-of select="@from" /></a></xsl:when>
				<xsl:otherwise><xsl:value-of select="@from" /></xsl:otherwise>
			</xsl:choose>
			TO 
			<xsl:choose>
				<xsl:when test="@to-function"><a href="#{@to-function}"><xsl:value-of select="@to" /></a></xsl:when>
				<xsl:otherwise><xsl:value-of select="@to" /></xsl:otherwise>
			</xsl:choose>
			<xsl:if test="@by">
				BY
				<xsl:choose>
					<xsl:when test="@by-function"><a href="#{@by-function}"><xsl:value-of select="@by" /></a></xsl:when>
					<xsl:otherwise><xsl:value-of select="@by" /></xsl:otherwise>
				</xsl:choose>
			</xsl:if>
			<ol><xsl:apply-templates /></ol>
		</li>
		<li>END-FOR</li>
	</xsl:template>
	
	<!-- for-each -->
	<xsl:template match="for-each">
		<li>FOR EACH <xsl:value-of select="@element" /> IN 
			<xsl:choose>
				<xsl:when test="@function"><a href="#{@function}"><xsl:value-of select="@array" /></a></xsl:when>
				<xsl:otherwise><xsl:value-of select="@array" /></xsl:otherwise>
			</xsl:choose>
			<ol><xsl:apply-templates /></ol>
		</li>
		<li>END-FOR</li>
	</xsl:template>
	
	<!-- cycle -->
	<xsl:template match="cycle">
		<li>CYCLE (PREPARATORY STEP : 
			<xsl:choose>
				<xsl:when test="@preparatory-function"><a href="#{@preparatory-function}"><xsl:value-of select="@preparatory" /></a>;</xsl:when>
				<xsl:otherwise><xsl:value-of select="@preparatory" />;</xsl:otherwise>
			</xsl:choose>
			CONDITION : 
			<xsl:choose>
				<xsl:when test="@condition-function"><a href="#{@condition-function}"><xsl:value-of select="@condition" /></a>;</xsl:when>
				<xsl:otherwise><xsl:value-of select="@condition" />;</xsl:otherwise>
			</xsl:choose>
			CONCLUDING STEP : 
			<xsl:choose>
				<xsl:when test="@concluding-function"><a href="#{@concluding-function}"><xsl:value-of select="@concluding" /></a>)</xsl:when>
				<xsl:otherwise><xsl:value-of select="@concluding" />)</xsl:otherwise>
			</xsl:choose>
			<ol><xsl:apply-templates /></ol>
		</li>
		<li>END-CYCLE</li>
	</xsl:template>
	
	<!-- switch -->
	<xsl:template match="switch">
		<li>SWITCH (VARIABLE : 
			<xsl:choose>
				<xsl:when test="@function"><a href="#{@function}"><xsl:value-of select="@variable" /></a>)</xsl:when>
				<xsl:otherwise><xsl:value-of select="@variable" />)</xsl:otherwise>
			</xsl:choose>
			<xsl:apply-templates select="case" />
			<xsl:apply-templates select="default" />
		</li>
		<li>END-SWITCH</li>
	</xsl:template>
	<xsl:template match="case">
		<li>CASE <xsl:value-of select="@value" /> :
			<ol><xsl:apply-templates /></ol>
		</li>
	</xsl:template>
	<xsl:template match="default">
		<li>DEFAULT :
			<ol><xsl:apply-templates /></ol>
		</li>
	</xsl:template>
	
	<!-- call -->
	<xsl:template match="call">
		<xsl:variable name="url" select="'#' + @function" as="xs:string" />
		<xsl:choose>
			<xsl:when test="@document">
				<xsl:choose>
					<xsl:when test="@out = 'file'">
						<a href="{@document}.html#{@function}"><xsl:value-of select="." /></a>
					</xsl:when>
					<xsl:otherwise>
						<a href="{@document}.xml#{@function}"><xsl:value-of select="." /></a>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<a href="#{@function}"><xsl:value-of select="." /></a>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- assert -->
	<xsl:template match="assert">
		<li>ASSERT THAT 
			<xsl:choose>
				<xsl:when test="@function"><a href="#{@function}"><xsl:value-of select="@actual" /></a></xsl:when>
				<xsl:otherwise><xsl:value-of select="@actual" /></xsl:otherwise>
			</xsl:choose>
			EQUALS TO
			<xsl:value-of select="@expected" />
			<xsl:if test="@delta">
				DESPITE 
				<xsl:value-of select="@delta" />
			</xsl:if>
		</li>
	</xsl:template>
	
	<!-- assert-true -->
	<xsl:template match="assert-true">
		<li>ASSERT THAT 
			<xsl:choose>
				<xsl:when test="@function"><a href="#{@function}"><xsl:value-of select="@assertion" /></a></xsl:when>
				<xsl:otherwise><xsl:value-of select="@assertion" /></xsl:otherwise>
			</xsl:choose>
		</li>
	</xsl:template>
		
	<!-- case-when -->
	<xsl:template match="case-when">
		<em class="comment"> { CASE WHEN <xsl:value-of select="." /> } </em>
	</xsl:template>
		
	<!-- label -->
	<xsl:template match="label">
		<li>LABEL <xsl:value-of select="@name" /> : </li>
	</xsl:template>
	
	<!-- break -->
	<xsl:template match="break">
		<li>BREAK <xsl:value-of select="@label" /></li>
	</xsl:template>
	
	<!-- continue -->
	<xsl:template match="continue">
		<li>CONTINUE <xsl:value-of select="@label" /></li>
	</xsl:template>
	
	<!-- return -->
	<xsl:template match="return">
		<li>RETURN 
			<xsl:choose>
				<xsl:when test="@function"><a href="#{@function}"><xsl:value-of select="@value" /></a></xsl:when>
				<xsl:otherwise><xsl:value-of select="@value" /></xsl:otherwise>
			</xsl:choose>
		</li>
	</xsl:template>
	
	<!-- throw -->
	<xsl:template match="throw">
		<li>THROW 
			<xsl:if test="@exception">
				<xsl:value-of select="@exception" />
			</xsl:if>
			:
			<xsl:choose>
				<xsl:when test="@class"><a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
				<xsl:otherwise><xsl:value-of select="@type" /></xsl:otherwise>
			</xsl:choose>
		</li>
	</xsl:template>	
	
	<!-- resolve -->
	<xsl:template match="resolve">
		<li>RESOLVE
			<ol>
				<xsl:apply-templates select="try" />
				<xsl:apply-templates select="catch" />
				<xsl:apply-templates select="finally" />
			</ol>
		</li>
		<li>END-RESOLVE</li>
	</xsl:template>
	
	<!-- try -->
	<xsl:template match="try">
		<li>TRY
			<ol><xsl:apply-templates /></ol>
		</li>
		<li>END-TRY</li>
	</xsl:template>
	
	<!-- catch -->
	<xsl:template match="catch">
		<li>CATCH 
			(
			<xsl:if test="@exception">
				<xsl:value-of select="@exception" />
			</xsl:if>
			:
			<xsl:choose>
				<xsl:when test="@class"><xsl:value-of select="@type" /></xsl:when>
				<xsl:otherwise><xsl:value-of select="@type" /></xsl:otherwise>
			</xsl:choose>
			)
			<ol><xsl:apply-templates /></ol>
		</li>
		<li>END-CATCH</li>
	</xsl:template>
	
	<!-- finally -->
	<xsl:template match="finally">
		<li>FINALLY
			<ol><xsl:apply-templates /></ol>
		</li>
		<li>END-FINALLY</li>
	</xsl:template>
	
	<!-- interaction -->
	<xsl:template match="interaction">
		<li>
			<span id="{generate-id(.)}-action"><a href="javascript:expandStep('{generate-id(.)}')">SHOW <xsl:value-of select="@dialog" /> DIALOG</a></span> 
			<ol id="{generate-id(.)}-detailing" style="display:none">
				<li>
					INTERACTION WITH 
					<xsl:choose>
						<xsl:when test="@screen"><a href="#{@screen}"><xsl:value-of select="@dialog" /></a></xsl:when>
						<xsl:otherwise><xsl:value-of select="@dialog" /></xsl:otherwise>
					</xsl:choose>
					DIALOG
					<ol>
						<xsl:apply-templates select="choice" />
					</ol>
				</li>
				<li>END-INTERACTION</li>
			</ol>
		</li>
	</xsl:template>

	<!-- choice -->
	<xsl:template match="choice">
		<li>CHOICE WHEN <xsl:value-of select="@event" />
			<ol><xsl:apply-templates /></ol>
		</li>
		<li>END-CHOICE</li>
	</xsl:template>
	
	<!-- with -->
	<xsl:template match="with">
		<li>WITH <xsl:value-of select="@objects" /> DO
			<ol><xsl:apply-templates /></ol>
		</li>
		<li>END-WITH</li>
	</xsl:template>
	
	<!-- increase -->
	<xsl:template match="increase">
		<li>
			<xsl:choose>
				<xsl:when test="@by">
					INCREASE <xsl:value-of select="@variable"/> BY
					<xsl:choose>
						<xsl:when test="@by-function"><a href="#{@by-function}"><xsl:value-of select="@by" /></a></xsl:when>
						<xsl:otherwise><xsl:value-of select="@by" /></xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					INCREMENT <xsl:value-of select="@variable"/>
				</xsl:otherwise>
			</xsl:choose>
		</li>		
	</xsl:template>
	
	<!-- decrease -->
	<xsl:template match="decrease">
		<li>
			<xsl:choose>
				<xsl:when test="@by">
					DECREASE <xsl:value-of select="@variable"/> BY
					<xsl:choose>
						<xsl:when test="@by-function"><a href="#{@by-function}"><xsl:value-of select="@by" /></a></xsl:when>
						<xsl:otherwise><xsl:value-of select="@by" /></xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>	
					DECREMENT <xsl:value-of select="@variable"/>
				</xsl:otherwise>
			</xsl:choose>
		</li>
	</xsl:template>
	
	<!-- perform-operations -->
	<xsl:template match="perform-operations">
		<li>PERFORM OPERATIONS</li>
	</xsl:template>
	
	<!-- comment -->
	<xsl:template match="comment">
		<em class="comment"> { <xsl:value-of select="." /> } </em>
	</xsl:template> 
	
</xsl:stylesheet>
