<?xml version="1.0" encoding="UTF-8"?>
<!-- Copyright (C) 2012 Oleh Radvanskyj -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions">
	<xsl:output method="html" encoding="UTF-8" />
	
	<!-- headers of functions -->
	
	<xsl:template match="function" mode="function_header">
		Function : <xsl:value-of select="@name" />
		<xsl:if test="parameterized-types">
		&lt;
		<xsl:for-each select="parameterized-types/parameterized-type">
			<xsl:value-of select="@name" /> 
			<xsl:if test="@type">
				<xsl:choose>
					<xsl:when test="@class"> extends <a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
					<xsl:otherwise> extends <xsl:value-of select="@type" /></xsl:otherwise>
				</xsl:choose>
			</xsl:if>
		 	<xsl:if test="not(position() = last())"> , </xsl:if>
		</xsl:for-each>
		&gt;
		</xsl:if>
		(
		<xsl:if test="parameters">
			<ul class="parameters">
				<xsl:for-each select="parameters/parameter">
					<li>
						<xsl:value-of select="@name" />
						:
						<xsl:choose>
							<xsl:when test="@primitive">
								<xsl:choose>
									<xsl:when test="@primitive = 'integer'">INTEGER</xsl:when>
									<xsl:when test="@primitive = 'real'">REAL</xsl:when>
									<xsl:when test="@primitive = 'pointer'">POINTER</xsl:when>
									<xsl:when test="@primitive = 'character'">CHARACTER</xsl:when>
									<xsl:when test="@primitive = 'string'">STRING</xsl:when>
									<xsl:when test="@primitive = 'natural'">NATURAL</xsl:when>
									<xsl:when test="@primitive = 'boolean'">BOOLEAN</xsl:when>
									<xsl:when test="@primitive = 'variant'">VARIANT</xsl:when>
								</xsl:choose>
							</xsl:when>
							<xsl:otherwise>
								<xsl:choose>
									<xsl:when test="@class"><a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
									<xsl:otherwise><xsl:value-of select="@type" /></xsl:otherwise>
								</xsl:choose>
							</xsl:otherwise>
						</xsl:choose>
						<xsl:if test="not(position() = last())"> ; </xsl:if>
					</li>
				</xsl:for-each>
			</ul>
		</xsl:if>
		)
		<xsl:if test="returns">
			: 
			<xsl:choose>
				<xsl:when test="returns/@primitive">
					<xsl:choose>
						<xsl:when test="returns/@primitive = 'integer'">INTEGER</xsl:when>
						<xsl:when test="returns/@primitive = 'real'">REAL</xsl:when>
						<xsl:when test="returns/@primitive = 'pointer'">POINTER</xsl:when>
						<xsl:when test="returns/@primitive = 'character'">CHARACTER</xsl:when>
						<xsl:when test="returns/@primitive = 'string'">STRING</xsl:when>
						<xsl:when test="returns/@primitive = 'natural'">NATURAL</xsl:when>
						<xsl:when test="returns/@primitive = 'boolean'">BOOLEAN</xsl:when>
						<xsl:when test="returns/@primitive = 'variant'">VARIANT</xsl:when>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:choose>
						<xsl:when test="returns/@class"><a href="#{returns/@class}"><xsl:value-of select="returns/@type" /></a></xsl:when>
						<xsl:otherwise><xsl:value-of select="returns/@type" /></xsl:otherwise>
					</xsl:choose> 
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>
	</xsl:template>	
	
	<xsl:template match="constructor" mode="constructor_header">
		<xsl:choose>
			<xsl:when test="@visibility = 'public'"> PUBLIC </xsl:when>
			<xsl:when test="@visibility = 'protected'"> PROTECTED </xsl:when>
			<xsl:when test="@visibility = 'private'"> PRIVATE </xsl:when>
			<xsl:when test="@visibility = 'package'"> PACKAGE </xsl:when>
		</xsl:choose>
		<xsl:choose>
			<xsl:when test="@name"> <xsl:value-of select="@name" /> </xsl:when>
			<xsl:otherwise> constructor </xsl:otherwise>
		</xsl:choose>
		<xsl:if test="parameterized-types">
		&lt;
		<xsl:for-each select="parameterized-types/parameterized-type">
			<xsl:value-of select="@name" /> 
			<xsl:if test="@type">
				<xsl:choose>
					<xsl:when test="@class"> extends <a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
					<xsl:otherwise> extends <xsl:value-of select="@type" /></xsl:otherwise>
				</xsl:choose>
			</xsl:if>
		 	<xsl:if test="not(position() = last())"> , </xsl:if>
		</xsl:for-each>
		&gt;
		</xsl:if>
		(
		<xsl:if test="parameters">
			<ul class="parameters">
				<xsl:for-each select="parameters/parameter">
					<li>
						<xsl:value-of select="@name" />
						<xsl:choose>
							<xsl:when test="@primitive"> : 
								<xsl:choose>
									<xsl:when test="@primitive = 'integer'">INTEGER</xsl:when>
									<xsl:when test="@primitive = 'real'">REAL</xsl:when>
									<xsl:when test="@primitive = 'pointer'">POINTER</xsl:when>
									<xsl:when test="@primitive = 'character'">CHARACTER</xsl:when>
									<xsl:when test="@primitive = 'string'">STRING</xsl:when>
									<xsl:when test="@primitive = 'natural'">NATURAL</xsl:when>
									<xsl:when test="@primitive = 'boolean'">BOOLEAN</xsl:when>
									<xsl:when test="@primitive = 'variant'">VARIANT</xsl:when>
								</xsl:choose>
							</xsl:when>
							<xsl:otherwise>
								<xsl:choose>
									<xsl:when test="@class"> : <a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
									<xsl:otherwise> : <xsl:value-of select="@type" /></xsl:otherwise>
								</xsl:choose>
							</xsl:otherwise>
						</xsl:choose>
						<xsl:if test="not(position() = last())"> ; </xsl:if>
					</li>
				</xsl:for-each>
			</ul>
		</xsl:if>
		)
	</xsl:template>
	
	<xsl:template match="operations/constructor">
		<li>
			<xsl:choose>
				<xsl:when test="@id">	
					<h4 id="{@id}" onclick="toggleTag('{generate-id(.)}')"><xsl:apply-templates select="." mode="constructor_header" /></h4>
				</xsl:when>
				<xsl:otherwise> 
					<h4 id="{generate-id(.)}-header" onclick="toggleTag('{generate-id(.)}')"><xsl:apply-templates select="." mode="constructor_header" /></h4>
				</xsl:otherwise>
			</xsl:choose>
			<div id="{generate-id(.)}" style="display:none" class="function">
				<xsl:apply-templates select="description" />
				<xsl:apply-templates select="preconditions" />
				<xsl:apply-templates select="postconditions" />
				<xsl:apply-templates select="parameters" />
				<xsl:apply-templates select="exceptions" />
				<xsl:apply-templates select="classes" />
				<xsl:apply-templates select="types" />
				<xsl:apply-templates select="constants" />
				<xsl:apply-templates select="shared-data" />
				<xsl:apply-templates select="data" />
				<xsl:apply-templates select="functions/function" />
				<xsl:apply-templates select="aspects" />
				<xsl:apply-templates select="sequence" />
				<xsl:apply-templates select="algorithm" />
				<xsl:apply-templates select="diagrams" />
				<xsl:apply-templates select="test-procedures/test-procedure" />
			</div>			
		</li>
	</xsl:template>
	
	<xsl:template match="destructor" mode="destructor_header">
		<xsl:choose>
			<xsl:when test="@name"> <xsl:value-of select="@name" /> </xsl:when>
			<xsl:otherwise> destructor </xsl:otherwise>
		</xsl:choose>
		<xsl:if test="parameterized-types">
		&lt;
		<xsl:for-each select="parameterized-types/parameterized-type">
			<xsl:value-of select="@name" /> 
			<xsl:if test="@type">
				<xsl:choose>
					<xsl:when test="@class"> extends <a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
					<xsl:otherwise> extends <xsl:value-of select="@type" /></xsl:otherwise>
				</xsl:choose>
			</xsl:if>
		 	<xsl:if test="not(position() = last())"> , </xsl:if>
		</xsl:for-each>
		&gt;
		</xsl:if>
		(
		<xsl:if test="parameters">
			<ul class="parameters">
				<xsl:for-each select="parameters/parameter">
					<li>
						<xsl:value-of select="@name" />
						<xsl:choose>
							<xsl:when test="@primitive"> : 
								<xsl:choose>
									<xsl:when test="@primitive = 'integer'">INTEGER</xsl:when>
									<xsl:when test="@primitive = 'real'">REAL</xsl:when>
									<xsl:when test="@primitive = 'pointer'">POINTER</xsl:when>
									<xsl:when test="@primitive = 'character'">CHARACTER</xsl:when>
									<xsl:when test="@primitive = 'string'">STRING</xsl:when>
									<xsl:when test="@primitive = 'natural'">NATURAL</xsl:when>
									<xsl:when test="@primitive = 'boolean'">BOOLEAN</xsl:when>
									<xsl:when test="@primitive = 'variant'">VARIANT</xsl:when>
								</xsl:choose>
							</xsl:when>
							<xsl:otherwise>
								<xsl:choose>
									<xsl:when test="@class"> : <a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
									<xsl:otherwise> : <xsl:value-of select="@type" /></xsl:otherwise>
								</xsl:choose>
							</xsl:otherwise>
						</xsl:choose>
						<xsl:if test="not(position() = last())"> ; </xsl:if>
					</li>
				</xsl:for-each>
			</ul>
		</xsl:if>
		)
	</xsl:template>
		
	<xsl:template match="operations/destructor">
		<li>
			<xsl:choose>
				<xsl:when test="@id">	
					<h4 id="{@id}" onclick="toggleTag('{generate-id(.)}')"><xsl:apply-templates select="." mode="destructor_header" /></h4>
				</xsl:when>
				<xsl:otherwise> 
					<h4 id="{generate-id(.)}-header" onclick="toggleTag('{generate-id(.)}')"><xsl:apply-templates select="." mode="destructor_header" /></h4>
				</xsl:otherwise>
			</xsl:choose>
			<div id="{generate-id(.)}" style="display:none" class="function">
				<xsl:apply-templates select="description" />
				<xsl:apply-templates select="preconditions" />
				<xsl:apply-templates select="postconditions" />
				<xsl:apply-templates select="parameters" />
				<xsl:apply-templates select="exceptions" />
				<xsl:apply-templates select="classes" />
				<xsl:apply-templates select="types" />
				<xsl:apply-templates select="constants" />
				<xsl:apply-templates select="shared-data" />
				<xsl:apply-templates select="data" />
				<xsl:apply-templates select="functions/function" />
				<xsl:apply-templates select="aspects" />
				<xsl:apply-templates select="sequence" />
				<xsl:apply-templates select="algorithm" />
				<xsl:apply-templates select="diagrams" />
				<xsl:apply-templates select="test-procedures/test-procedure" />
			</div>			
		</li>
	</xsl:template>
	
	<xsl:template match="method" mode="method_header">
		<xsl:if test="@overridden = 'true'"> OVERRIDDEN </xsl:if>
		<xsl:if test="@static or @visibility or @abstract or @modifiers">
			<xsl:if test="@static = 'true'"> STATIC </xsl:if>
			<xsl:choose>
				<xsl:when test="@visibility = 'public'"> PUBLIC </xsl:when>
				<xsl:when test="@visibility = 'protected'"> PROTECTED </xsl:when>
				<xsl:when test="@visibility = 'private'"> PRIVATE </xsl:when>
				<xsl:when test="@visibility = 'package'"> PACKAGE </xsl:when>
			</xsl:choose>
			<xsl:if test="@abstract = 'true'"> ABSTRACT </xsl:if>
			<xsl:value-of select="@modifiers" />
		</xsl:if>
		<xsl:value-of select="@name" />
		<xsl:if test="parameterized-types">
		&lt;
		<xsl:for-each select="parameterized-types/parameterized-type">
			<xsl:value-of select="@name" /> 
			<xsl:if test="@type">
				<xsl:choose>
					<xsl:when test="@class"> extends <a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
					<xsl:otherwise> extends <xsl:value-of select="@type" /></xsl:otherwise>
				</xsl:choose>
			</xsl:if>
		 	<xsl:if test="not(position() = last())"> , </xsl:if>
		</xsl:for-each>
		&gt;
		</xsl:if>
		(
		<xsl:if test="parameters">
			<ul class="parameters">
				<xsl:for-each select="parameters/parameter">
					<li>
						<xsl:value-of select="@name" />
						<xsl:choose>
							<xsl:when test="@primitive"> : 
								<xsl:choose>
									<xsl:when test="@primitive = 'integer'">INTEGER</xsl:when>
									<xsl:when test="@primitive = 'real'">REAL</xsl:when>
									<xsl:when test="@primitive = 'pointer'">POINTER</xsl:when>
									<xsl:when test="@primitive = 'character'">CHARACTER</xsl:when>
									<xsl:when test="@primitive = 'string'">STRING</xsl:when>
									<xsl:when test="@primitive = 'natural'">NATURAL</xsl:when>
									<xsl:when test="@primitive = 'boolean'">BOOLEAN</xsl:when>
									<xsl:when test="@primitive = 'variant'">VARIANT</xsl:when>
								</xsl:choose>
							</xsl:when>
							<xsl:otherwise>
								<xsl:choose>
									<xsl:when test="@class"> : <a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
									<xsl:otherwise> : <xsl:value-of select="@type" /></xsl:otherwise>
								</xsl:choose>
							</xsl:otherwise>
						</xsl:choose>
						<xsl:if test="not(position() = last())"> ; </xsl:if>
					</li>
				</xsl:for-each>
			</ul>
		</xsl:if>
		)
		<xsl:if test="returns">
			: 
			<xsl:choose>
				<xsl:when test="returns/@primitive">
					<xsl:choose>
						<xsl:when test="returns/@primitive = 'integer'">INTEGER</xsl:when>
						<xsl:when test="returns/@primitive = 'real'">REAL</xsl:when>
						<xsl:when test="returns/@primitive = 'pointer'">POINTER</xsl:when>
						<xsl:when test="returns/@primitive = 'character'">CHARACTER</xsl:when>
						<xsl:when test="returns/@primitive = 'string'">STRING</xsl:when>
						<xsl:when test="returns/@primitive = 'natural'">NATURAL</xsl:when>
						<xsl:when test="returns/@primitive = 'boolean'">BOOLEAN</xsl:when>
						<xsl:when test="returns/@primitive = 'variant'">VARIANT</xsl:when>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:choose>
						<xsl:when test="returns/@class"><a href="#{returns/@class}"><xsl:value-of select="returns/@type" /></a></xsl:when>
						<xsl:otherwise><xsl:value-of select="returns/@type" /></xsl:otherwise>
					</xsl:choose> 
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>	
	</xsl:template>
	
	<xsl:template match="interface/method">
		<li><xsl:apply-templates select="." mode="method_header" /></li>
	</xsl:template>
			
	<xsl:template match="operations/method">
		<li>
			<xsl:choose>
				<xsl:when test="@id">	
					<xsl:choose>
						<xsl:when test="@function">
							<a href="#{@function}"><h4 id="{@id}" onclick="toggleTag('{generate-id(.)}')"><xsl:apply-templates select="." mode="method_header" /></h4></a>
						</xsl:when>
						<xsl:otherwise>
							<h4 id="{@id}" onclick="toggleTag('{generate-id(.)}')"><xsl:apply-templates select="." mode="method_header" /></h4>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise> 
					<xsl:choose>
						<xsl:when test="@function">
							<a href="#{@function}"><h4 id="{generate-id(.)}-header" onclick="toggleTag('{generate-id(.)}')"><xsl:apply-templates select="." mode="method_header" /></h4></a>
						</xsl:when>
						<xsl:otherwise>
							<h4 id="{generate-id(.)}-header" onclick="toggleTag('{generate-id(.)}')"><xsl:apply-templates select="." mode="method_header" /></h4>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
			<div id="{generate-id(.)}" style="display:none" class="function">
				<xsl:apply-templates select="description" />
				<xsl:apply-templates select="preconditions" />
				<xsl:apply-templates select="postconditions" />
				<xsl:apply-templates select="parameters" />
				<xsl:apply-templates select="returns" />
				<xsl:apply-templates select="exceptions" />
				<xsl:apply-templates select="classes" />
				<xsl:apply-templates select="types" />
				<xsl:apply-templates select="constants" />
				<xsl:apply-templates select="shared-data" />
				<xsl:apply-templates select="data" />
				<xsl:apply-templates select="functions/function" />
				<xsl:apply-templates select="aspects" />
				<xsl:apply-templates select="sequence" />
				<xsl:apply-templates select="algorithm" />
				<xsl:apply-templates select="diagrams" />
				<xsl:apply-templates select="test-procedures/test-procedure" />
			</div>			
		</li>
	</xsl:template>
	
	<!-- content of functions -->
	
	<xsl:template match="function/description | test-procedure/description | constructor/description | destructor/description | method/description">
		<h4>Description</h4>
		<p class="indent"><xsl:apply-templates /></p>
	</xsl:template>

	<xsl:template match="function/parameters | test-procedure/parameters | constructor/parameters | destructor/parameters | method/parameters">
		<h4>Parameters</h4>
		<ul><xsl:apply-templates /></ul>
	</xsl:template>
	
	<xsl:template match="parameter">
		<li>
			<xsl:choose>
				<xsl:when test="@modifier = 'input'">INPUT </xsl:when>
				<xsl:when test="@modifier = 'output'">OUTPUT </xsl:when>
				<xsl:when test="@modifier = 'mutable'">MUTABLE </xsl:when>
			</xsl:choose>
			<xsl:value-of select="@name" />
			<xsl:choose>
				<xsl:when test="@primitive"> : 
					<xsl:choose>
						<xsl:when test="@primitive = 'integer'">INTEGER</xsl:when>
						<xsl:when test="@primitive = 'real'">REAL</xsl:when>
						<xsl:when test="@primitive = 'pointer'">POINTER</xsl:when>
						<xsl:when test="@primitive = 'character'">CHARACTER</xsl:when>
						<xsl:when test="@primitive = 'string'">STRING</xsl:when>
						<xsl:when test="@primitive = 'natural'">NATURAL</xsl:when>
						<xsl:when test="@primitive = 'boolean'">BOOLEAN</xsl:when>
						<xsl:when test="@primitive = 'variant'">VARIANT</xsl:when>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:choose>
						<xsl:when test="@class"> : <a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
						<xsl:otherwise> : <xsl:value-of select="@type" /></xsl:otherwise>
					</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:if test="@value"> = <xsl:value-of select="@value" /></xsl:if>
			<xsl:if test="@description"><em class="comment"> { <xsl:value-of select="@description" /> } </em></xsl:if>
		</li>
	</xsl:template>

	<xsl:template match="function/returns | method/returns">
		<h4>Returns</h4>
		<p class="indent">
			<xsl:choose>
				<xsl:when test="@primitive">
					<xsl:choose>
						<xsl:when test="@primitive = 'integer'">INTEGER</xsl:when>
						<xsl:when test="@primitive = 'real'">REAL</xsl:when>
						<xsl:when test="@primitive = 'pointer'">POINTER</xsl:when>
						<xsl:when test="@primitive = 'character'">CHARACTER</xsl:when>
						<xsl:when test="@primitive = 'string'">STRING</xsl:when>
						<xsl:when test="@primitive = 'natural'">NATURAL</xsl:when>
						<xsl:when test="@primitive = 'boolean'">BOOLEAN</xsl:when>
						<xsl:when test="@primitive = 'variant'">VARIANT</xsl:when>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:choose>
						<xsl:when test="@class"><a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
						<xsl:otherwise><xsl:value-of select="@type" /></xsl:otherwise>
					</xsl:choose> 
				</xsl:otherwise>
			</xsl:choose>
			<xsl:if test="@definition"> = <xsl:value-of select="@definition" /></xsl:if>
			<xsl:if test="@description"><em class="comment"> { <xsl:value-of select="@description" /> } </em></xsl:if>
		</p>
	</xsl:template>

	<xsl:template match="function/exceptions | test-procedure/exceptions | constructor/exceptions | destructor/exceptions | method/exceptions">
		<h4>Exceptions</h4>
		<ul><xsl:apply-templates /></ul>
	</xsl:template>

	<xsl:template match="function/shared-data | test-procedure/shared-data | constructor/shared-data | destructor/shared-data | method/shared-data">
		<h4>Shared data</h4>
		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="input">
		<p>INPUT : </p>
		<ul><xsl:apply-templates /></ul>
	</xsl:template>

	<xsl:template match="output">
		<p>OUTPUT : </p>
		<ul><xsl:apply-templates /></ul>
	</xsl:template>
	
	<xsl:template match="mutable">
		<p>MUTABLE : </p>
		<ul><xsl:apply-templates /></ul>
	</xsl:template>
	
	<xsl:template match="function/constants | test-procedure/constants | constructor/constants | destructor/constants | method/constants">
		<h4>Constants</h4>
		<ul><xsl:apply-templates /></ul>
	</xsl:template>

	<xsl:template match="function/data | test-procedure/data | constructor/data | destructor/data | method/data">
		<h4>Data</h4>
		<ul><xsl:apply-templates /></ul>
	</xsl:template>

	<xsl:template match="variable">
		<li id="@id">
			<xsl:if test="@static = 'true'"> STATIC </xsl:if>
			<xsl:choose>
				<xsl:when test="@refetence"><a href="#{@reference}"><xsl:value-of select="@name" /></a></xsl:when>
				<xsl:otherwise><xsl:value-of select="@name" /></xsl:otherwise>
			</xsl:choose>
			<xsl:choose>
				<xsl:when test="@primitive">
					<xsl:choose>
						<xsl:when test="@primitive = 'integer'"> : INTEGER</xsl:when>
						<xsl:when test="@primitive = 'real'"> : REAL</xsl:when>
						<xsl:when test="@primitive = 'pointer'"> : POINTER</xsl:when>
						<xsl:when test="@primitive = 'character'"> : CHARACTER</xsl:when>
						<xsl:when test="@primitive = 'string'"> : STRING</xsl:when>
						<xsl:when test="@primitive = 'natural'"> : NATURAL</xsl:when>
						<xsl:when test="@primitive = 'boolean'"> : BOOLEAN</xsl:when>
						<xsl:when test="@primitive = 'variant'"> : VARIANT</xsl:when>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:choose>
						<xsl:when test="@class"> : <a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
						<xsl:otherwise> : <xsl:value-of select="@type" /></xsl:otherwise>
					</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:if test="@value"> = <xsl:value-of select="@value" /></xsl:if>
			<xsl:if test="@purpose"><em class="comment"> { PURPOSE : <xsl:value-of select="@purpose" /> } </em></xsl:if>
			<xsl:if test="@description"><em class="comment"> { <xsl:value-of select="@description" /> } </em></xsl:if>
		</li>
	</xsl:template>
	
	<xsl:template match="constant">
		<li>
			<xsl:value-of select="@name" /> = <xsl:value-of select="@value" />
			<xsl:if test="@description"><em class="comment"> { <xsl:value-of select="@description" /> } </em></xsl:if>
		</li>
	</xsl:template>
	
	<xsl:template match="function" mode="inner_function">
		<xsl:choose>
			<xsl:when test="@id">
				<h4 id="{@id}" class="inner-function" onclick="toggleTag('{generate-id(.)}')">
					<xsl:apply-templates select="." mode="function_header" />
				</h4>
			</xsl:when>
			<xsl:otherwise>
				<h4 id="{generate-id(.)}-header" class="inner-function" onclick="toggleTag('{generate-id(.)}')">
					<xsl:apply-templates select="." mode="function_header" />
				</h4>
			</xsl:otherwise>
		</xsl:choose>
		<div id="{generate-id(.)}" style="display:none" class="function">
			<xsl:apply-templates select="description" />
			<xsl:apply-templates select="preconditions" />
			<xsl:apply-templates select="postconditions" />
			<xsl:apply-templates select="returns" />
			<xsl:apply-templates select="exceptions" />
			<xsl:apply-templates select="classes" />
			<xsl:apply-templates select="types" />
			<xsl:apply-templates select="constants" />
			<xsl:apply-templates select="shared-data" />
			<xsl:apply-templates select="data" />
			<xsl:apply-templates select="functions/function" />
			<xsl:apply-templates select="aspects" />
			<xsl:apply-templates select="sequence" />
			<xsl:apply-templates select="algorithm" />
			<xsl:apply-templates select="diagrams" />		
			<xsl:apply-templates select="test-procedures/test-procedure" />
		</div>
	</xsl:template>
	
	<xsl:template match="test-procedure" mode="inner_test_procedure">
		<xsl:choose>
			<xsl:when test="@id"><h4 id="{@id}" class="inner-function" onclick="toggleTag('{generate-id(.)}')">Test procedure : <xsl:value-of select="@name" /></h4></xsl:when>
			<xsl:otherwise><h4 id="{generate-id(.)}-header" class="inner-function" onclick="toggleTag('{generate-id(.)}')">Test procedure : <xsl:value-of select="@name" /></h4></xsl:otherwise>
		</xsl:choose>
		<div id="{generate-id(.)}" style="display:none" class="function">
			<xsl:apply-templates select="description" />
			<xsl:apply-templates select="preconditions" />
			<xsl:apply-templates select="postconditions" />
			<xsl:apply-templates select="exceptions" />
			<xsl:apply-templates select="classes" />
			<xsl:apply-templates select="types" />
			<xsl:apply-templates select="constants" />
			<xsl:apply-templates select="shared-data" />
			<xsl:apply-templates select="data" />
			<xsl:apply-templates select="functions/function" />
			<xsl:apply-templates select="aspects" />
			<xsl:apply-templates select="sequence" />
			<xsl:apply-templates select="algorithm" />
			<xsl:apply-templates select="diagrams" />
			<xsl:apply-templates select="test-procedures/test-procedure" />					
		</div>
	</xsl:template>
		
	<xsl:template match="function/functions/function | test-procedure/functions/function | constructor/functions/function | destructor/functions/function | method/functions/function">
		<xsl:apply-templates select="." mode="inner_function" />
	</xsl:template>
	
	<xsl:template match="function/test-procedures/test-procedure | constructor/test-procedures/test-procedure | destructor/test-procedures/test-procedure | method/test-procedures/test-procedure | test-procedure/test-procedures/test-procedure | class/test-procedures/test-procedure | feature/test-procedures/test-procedure">
		<xsl:apply-templates select="." mode="inner_test_procedure" />
	</xsl:template>
	
	<xsl:template match="function/sequence | constructor/sequence | destructor/sequence | method/sequence | test-procedure/sequence">
		<h4 onclick="toggleTag('{generate-id(.)}')">Sequence</h4>
		<div id="{generate-id(.)}" class="indent" style="display:none">
			<xsl:apply-templates select="application" />
		</div>
	</xsl:template>
	
	<xsl:template match="sequence">
		<h4 onclick="toggleTag('{generate-id(.)}')">Sequence</h4>
		<div id="{generate-id(.)}" class="indent">
			<xsl:apply-templates select="application" />
		</div>
	</xsl:template>
	
	<xsl:template match="application">
		<h4 onclick="toggleTag('{generate-id(.)}')">Application</h4>
		<div id="{generate-id(.)}" class="indent">
			<xsl:if test="requirement">
				<xsl:choose>
					<xsl:when test="count(requirement) > 1"><h4>Requirements</h4></xsl:when>
					<xsl:otherwise><h4>Requirement</h4></xsl:otherwise>
				</xsl:choose>
				<div class="indent">
					<ul class="requirements">
						<xsl:for-each select="requirement">
							<li>ASSERT THAT <xsl:value-of select="@assertion" /></li>
						</xsl:for-each>
					</ul>
				</div>
			</xsl:if>
			<xsl:apply-templates select="sequence" />
			<xsl:apply-templates select="aspects" />
			<xsl:apply-templates select="algorithm" />
		</div>
	</xsl:template>
	
	<xsl:template match="function/aspects | constructor/aspects | destructor/aspects | method/aspects | test-procedure/aspects">
		<h4 onclick="toggleTag('{generate-id(.)}')">Aspects</h4>
		<div id="{generate-id(.)}" class="indent" style="display:none">
			<xsl:apply-templates />
		</div>
	</xsl:template>

	<xsl:template match="aspects">
		<h4 onclick="toggleTag('{generate-id(.)}')">Aspects</h4>
		<div id="{generate-id(.)}" class="indent">
			<xsl:apply-templates />
		</div>
	</xsl:template>
	
	<xsl:template match="precedent">
		<h4 onclick="toggleTag('{generate-id(.)}')">Precedent</h4>
		<div id="{generate-id(.)}" class="indent">
			<xsl:apply-templates select="preconditions" />
			<xsl:apply-templates select="postconditions" />
			<xsl:apply-templates select="algorithm" />
		</div>
	</xsl:template>
	
	<xsl:template match="function/preconditions | constructor/preconditions | destructor/preconditions | method/preconditions | test-procedure/preconditions | precedent/preconditions">
		<h4>Preconditions</h4>
		<xsl:choose>
			<xsl:when test="condition">
				<div class="indent">
					<ul class="conditions"><xsl:apply-templates /></ul>
				</div>  
			</xsl:when>
			<xsl:otherwise>
				<p><xsl:apply-templates /></p>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="function/postconditions | constructor/postconditions | destructor/postconditions | method/postconditions | test-procedure/postconditions | precedent/postconditions">
		<h4>Postconditions</h4>
		<div class="indent">
			<xsl:choose>
				<xsl:when test="condition"><ul class="conditions"><xsl:apply-templates /></ul></xsl:when>
				<xsl:otherwise><xsl:apply-templates /></xsl:otherwise>
			</xsl:choose>
		</div>
	</xsl:template>

	<xsl:template match="reaction">
		<h4 onclick="toggleTag('{generate-id(.)}')">Reaction</h4>
		<div id="{generate-id(.)}" class="indent">
			<xsl:apply-templates select="state" />
			<xsl:apply-templates select="algorithm" />
		</div>
	</xsl:template>
	
	<xsl:template match="state">
		<h4>State</h4>
		<ul class="conditions">
			<xsl:for-each select="condition" />
		</ul>
	</xsl:template>
	
	<xsl:template match="condition">
		<li><xsl:value-of select="@assertion" />
			<xsl:if test="true()">
				( <xsl:apply-templates /> )
			</xsl:if>
		</li>
	</xsl:template>
	
	<xsl:template match="aspect">
		<h4 onclick="toggleTag('{generate-id(.)}')">Aspect</h4>
		<div id="{generate-id(.)}" class="indent">
			<xsl:choose>
				<xsl:when test="count(viewpoint) > 1"><h4>Viewpoints</h4></xsl:when>
				<xsl:otherwise><h4>Viewpoint</h4></xsl:otherwise>
			</xsl:choose>
			<xsl:choose>
				<xsl:when test="count(viewpoint) > 1">
					<ul class="viewpoints">
						<xsl:for-each select="viewpoint">
							<li>
								<xsl:choose>
									<xsl:when test="@responsibility"><a href="#{@responsibility}"><xsl:value-of select="viewpoint" /></a></xsl:when>
									<xsl:otherwise><xsl:value-of select="viewpoint" /></xsl:otherwise>
								</xsl:choose>
							</li>
						</xsl:for-each>
					</ul>
				</xsl:when>
				<xsl:otherwise>
					<xsl:choose>
						<xsl:when test="@responsibility"><p><a href="#{@responsibility}"><xsl:value-of select="viewpoint" /></a></p></xsl:when>
						<xsl:otherwise><p><xsl:value-of select="viewpoint" /></p></xsl:otherwise>
					</xsl:choose>				
				</xsl:otherwise>
			</xsl:choose>
			<xsl:apply-templates select="algorithm" />
		</div>
	</xsl:template>

	<xsl:template match="function/algorithm | test-procedure/algorithm | constructor/algorithm | destructor/algorithm | method/algorithm | alternative-case/algorithm | application/algorithm | precedent/algorithm | reaction/algorithm | aspect/algorithm">
		<h4 onclick="toggleAlgorithm('{generate-id(.)}')">Algorithm</h4>
		<ol id="{generate-id(.)}" state="collapsed" class="algorithm"><xsl:apply-templates /></ol>
	</xsl:template>
	
</xsl:stylesheet>
