<?xml version="1.0" encoding="UTF-8"?>
<!-- Copyright (C) 2012 Oleh Radvanskyj -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions">
	<xsl:output method="html" encoding="UTF-8" indent="yes"/>
	
	<!-- overview -->
	
	<xsl:template match="pd-package">
		<table class="pd">
			<xsl:choose>
				<xsl:when test="@name"><caption id="{generate-id(.)}"><xsl:value-of select="@name" /></caption></xsl:when>
				<xsl:otherwise><caption id="{generate-id(.)}">PD</caption></xsl:otherwise>
			</xsl:choose>
			<thead>
				<tr>
					<th>Name</th>
					<th>Description</th>
				</tr>
			</thead>
			<tbody>
				<xsl:apply-templates select="pd" />
			</tbody>
		</table>
	</xsl:template>
	
	<xsl:template match="pd">
		<tr>
			<td><xsl:value-of select="name" /></td>
			<td><xsl:value-of select="description" /></td>
		</tr>
	</xsl:template>
	
	<xsl:template match="cd-package">
		<table class="cd">
			<xsl:choose>
				<xsl:when test="@name"><caption id="{generate-id(.)}"><xsl:value-of select="@name" /></caption></xsl:when>
				<xsl:otherwise><caption id="{generate-id(.)}">CD</caption></xsl:otherwise>
			</xsl:choose>
			<thead>
				<tr>
					<th>Name</th>
					<th>Description</th>
				</tr>
			</thead>
			<tbody>
				<xsl:apply-templates select="cd" />
			</tbody>
		</table>
	</xsl:template>
	
	<xsl:template match="cd-package/cd">
		<tr>
			<td><xsl:value-of select="name" /></td>
			<td><xsl:value-of select="description" /></td>
		</tr>
	</xsl:template>
	
	<xsl:template match="crc-package">
		<table class="crc">
			<xsl:choose>
				<xsl:when test="@name"><caption id="{generate-id(.)}"><xsl:value-of select="@name" /></caption></xsl:when>
				<xsl:otherwise><caption id="{generate-id(.)}">CRC</caption></xsl:otherwise>
			</xsl:choose>
			<thead>
				<tr>
					<th>Name</th>
					<th>Responsibilities</th>
					<th>Collaboration</th>
				</tr>
			</thead>
			<tbody>
				<xsl:apply-templates select="crc" />
			</tbody>
		</table>
	</xsl:template>

	<xsl:template match="crc">
		<tr>
			<td><xsl:apply-templates select="name" /></td>
			<td><xsl:apply-templates select="responsibilities" /></td>
			<td><xsl:apply-templates select="collaboration" /></td>
		</tr>
	</xsl:template>
	
	<xsl:template match="crc/name">
		<xsl:value-of select="." />
	</xsl:template>
	
	<xsl:template match="crc/responsibilities">
		<xsl:if test="not(responsibility)">
			<br />
		</xsl:if>
		<xsl:for-each select="responsibility">
			<xsl:value-of select="." /><br />
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template match="crc/collaboration">
		<xsl:if test="not(relationship)">
			<br />
		</xsl:if>
		<xsl:for-each select="relationship">
			<xsl:value-of select="." /><br />
		</xsl:for-each>
	</xsl:template>

	<!-- package -->

	<xsl:template match="package" mode="pd-overview">
		<xsl:if test="package">
			<table class="pd">
				<caption>Package - Description</caption>
				<thead>
					<tr>
						<th>Name</th>
						<th>Description</th>
					</tr>
				</thead>
				<tbody>
					<xsl:for-each select="package">
						<tr>
							<td>
								<xsl:choose>
									<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
									<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
								</xsl:choose>
							</td>
							<td>
								<xsl:value-of select="description" />
							</td>
						</tr>
					</xsl:for-each>
				</tbody>
			</table>
		</xsl:if>
	</xsl:template>

	<xsl:template match="package" mode="cd-overview">
		<xsl:if test="interface or class">
			<table class="cd">
				<caption>Class - Description</caption>
				<thead>
					<tr>
						<th>Name</th>
						<th>Description</th>
					</tr>
				</thead>
				<tbody>
					<xsl:for-each select="interface">
						<tr>
							<td>
								<xsl:choose>
									<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
									<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
								</xsl:choose>
							</td>
							<td><xsl:value-of select="description" /></td>
						</tr>
					</xsl:for-each>
					<xsl:for-each select="class">
						<tr>
							<td>
								<xsl:choose>
									<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
									<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
								</xsl:choose>
							</td>
							<td><xsl:value-of select="description" /></td>
						</tr>
					</xsl:for-each>
				</tbody>
			</table>				
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="package" mode="crc-overview">
		<xsl:if test="interface or class">
			<table class="crc">
				<caption>Class - Responsibilities - Collaboration</caption>
				<thead>
					<tr>
						<th>Name</th>
						<th>Responsibilities</th>
						<th>Collaboration</th>
					</tr>
				</thead>
				<tbody>
					<xsl:for-each select="interface">
						<tr>
							<td>
								<xsl:choose>
									<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
									<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
								</xsl:choose>
							</td>
							<td>
								<ul class="responsibilities">
									<xsl:for-each select="responsibility">
										<li><xsl:value-of select="." /></li>
									</xsl:for-each>
								</ul>
							</td>
							<td>
								<ul class="collaboration">
									<xsl:for-each select="implementation">
										<li>Implements
											<xsl:choose>
												<xsl:when test="@reference"><a href="#{@reference}"><xsl:value-of select="@interface" /></a></xsl:when>
												<xsl:otherwise><xsl:value-of select="@interface" /></xsl:otherwise>
											</xsl:choose>
										 </li>
									</xsl:for-each>
								</ul>
							</td>
						</tr>
					</xsl:for-each>
					<xsl:for-each select="class">
						<tr>
							<td>
								<xsl:choose>
									<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
									<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
								</xsl:choose>
							</td>
							<td>
								<ul class="responsibilities">
									<xsl:for-each select="responsibility">
										<li><xsl:value-of select="." /></li>
									</xsl:for-each>
								</ul>
							</td>
							<td>
								<ul class="collaboration">
									<xsl:for-each select="implementation">
										<li>Implements
											<xsl:choose>
												<xsl:when test="@reference"><a href="#{@reference}"><xsl:value-of select="@interface" /></a></xsl:when>
												<xsl:otherwise><xsl:value-of select="@interface" /></xsl:otherwise>
											</xsl:choose>
										 </li>
									</xsl:for-each>
									<xsl:for-each select="superclass">
										<li>Extends
											<xsl:choose>
												<xsl:when test="@class"><a href="#{@class}"><xsl:value-of select="@name" /></a></xsl:when>
												<xsl:otherwise><xsl:value-of select="@name" /></xsl:otherwise>
											</xsl:choose>
										 </li>
									</xsl:for-each>
								</ul>
							</td>
						</tr>
					</xsl:for-each>
				</tbody>
			</table>			
		</xsl:if>
	</xsl:template>	
	
</xsl:stylesheet>
