<?xml version="1.0" encoding="UTF-8"?>
<!-- Copyright (C) 2012 Oleh Radvanskyj -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions">
	<xsl:output method="html" encoding="UTF-8" />
	
	<!-- inner use case -->
	
	<xsl:template match="package/use-case">
		<h4 id="{generate-id(.)}">
			<xsl:choose>
				<xsl:when test="@type='abstract'">Abstract use case : </xsl:when>
				<xsl:when test="@type='concrete'">Concrete use case : </xsl:when>
				<xsl:otherwise>Use case : </xsl:otherwise>
			</xsl:choose>
			<span class="usecase-id"><xsl:value-of select="@name" /></span>
			<xsl:if test="@extends">
				extends <xsl:value-of select="@extends" />
			</xsl:if>			
		</h4>
		<div class="level_of_model">
			<xsl:apply-templates select="description" />
			<xsl:apply-templates select="goal" />
			<xsl:apply-templates select="concretization" />
			<xsl:apply-templates select="actors" />
			<xsl:apply-templates select="preconditions" />
			<xsl:apply-templates select="narrative" />
			<xsl:apply-templates select="postconditions" />
			<xsl:apply-templates select="alternative" />
			<xsl:apply-templates select="segment" />
			<xsl:apply-templates select="packages/package" />
			<xsl:apply-templates select="classes" />
			<xsl:apply-templates select="functions/function" />
			<xsl:apply-templates select="thread" />
			<xsl:apply-templates select="test-procedures/test-procedure" />
			<xsl:apply-templates select="test-thread" />
			<xsl:apply-templates select="artifacts" />
			<xsl:apply-templates select="gui/screen" />
			<xsl:apply-templates select="test-scenarios" />
			<xsl:apply-templates select="diagrams/diagram" />
		</div>
	</xsl:template>
		
  <xsl:template match="package/use-case/description">
    <h4>Description</h4>
    <p><xsl:apply-templates /></p>
  </xsl:template>
  
  <xsl:template match="package/use-case/goal">
	  <h4>Goal</h4>
	  <p><xsl:apply-templates /></p>
  </xsl:template>
  
  <xsl:template match="package/use-case/preconditions">
    <h4>Preconditions</h4>
	<xsl:choose>
		<xsl:when test="condition">
			<div class="indent">
				<ul class="conditions"><xsl:apply-templates /></ul>
			</div>  
		</xsl:when>
		<xsl:otherwise>
			<p><xsl:apply-templates /></p>
		</xsl:otherwise>
	</xsl:choose>
  </xsl:template>
  
  <xsl:template match="package/use-case/narrative">
	<h4 onclick="toggleAlgorithm('{generate-id(.)}')">Narrative</h4>
	<ol id="{generate-id(.)}" state="collapsed" class="algorithm"><xsl:apply-templates /></ol>
  </xsl:template>
  
  <xsl:template match="package/use-case/postconditions">
    <h4>Postconditions</h4>
	<xsl:choose>
		<xsl:when test="condition">
			<div class="indent">
				<ul class="conditions"><xsl:apply-templates /></ul>
			</div>  
		</xsl:when>
		<xsl:otherwise>
			<p><xsl:apply-templates /></p>
		</xsl:otherwise>
	</xsl:choose>
  </xsl:template>

  <xsl:template match="package/use-case/thread">
  	<xsl:choose>
		<xsl:when test="@id">
			<h4 id="{@id}" onclick="toggleAlgorithm('{generate-id(.)}')">Thread
				<xsl:if test="@name">
					: <xsl:value-of select="@name" />
				</xsl:if>
				<xsl:if test="@description">
					( <xsl:value-of select="@description" /> )
				</xsl:if>
			</h4>
		</xsl:when>
		<xsl:otherwise>
			<h4 onclick="toggleAlgorithm('{generate-id(.)}')">Thread
				<xsl:if test="@name">
					: <xsl:value-of select="@name" />
				</xsl:if>
				<xsl:if test="@description">
					( <xsl:value-of select="@description" /> )
				</xsl:if>
			</h4>
		</xsl:otherwise>
	</xsl:choose>
    <ol id="{generate-id(.)}" state="collapsed" class="algorithm"><xsl:apply-templates /></ol>
  </xsl:template>

  <xsl:template match="package/use-case/test-thread">
    <h4 onclick="toggleAlgorithm('{generate-id(.)}')">Test thread</h4>
    <ol id="{generate-id(.)}" state="collapsed" class="algorithm"><xsl:apply-templates /></ol>
  </xsl:template>

  <xsl:template match="package/use-case/artifacts">
    <h4>Artifacts</h4>
    <xsl:choose>
		<xsl:when test="artifact">
			<ul class="artifacts"><xsl:apply-templates /></ul>
		</xsl:when>
		<xsl:otherwise>
			<pre><xsl:apply-templates /></pre>
		</xsl:otherwise>
	</xsl:choose>
  </xsl:template>

</xsl:stylesheet>
