<?xml version="1.0" encoding="UTF-8"?>
<!-- Copyright (C) 2012 Oleh Radvanskyj -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions">
	<xsl:output method="html" encoding="UTF-8" />
	
	<!-- types -->

	<xsl:template match="function/types | test-procedure/types | constructor/types | destructor/types | method/types">
		<h4>Types</h4>
		<ul><xsl:apply-templates /></ul>
	</xsl:template>

	<xsl:template match="attributes/types">
		<xsl:apply-templates />
	</xsl:template>
	
	<xsl:template match="type">
		<li>
			<xsl:choose>
				<xsl:when test="@class"><a href="#{@class}"><xsl:value-of select="@name" /></a></xsl:when>
				<xsl:otherwise><xsl:value-of select="@name" /></xsl:otherwise>
			</xsl:choose> 
			<xsl:if test="@definition"> IS <xsl:value-of select="@definition" /></xsl:if>
			<xsl:if test="@description"><em class="comment"> { <xsl:value-of select="@description" /> } </em></xsl:if>
		</li>
	</xsl:template>
	
	<xsl:template match="primitive">
		<li>
			<xsl:value-of select="@name" /> IS 
			<xsl:choose>
				<xsl:when test="@type = 'integer'">INTEGER</xsl:when>
				<xsl:when test="@type = 'real'">REAL</xsl:when>
				<xsl:when test="@type = 'pointer'">POINTER</xsl:when>
				<xsl:when test="@type = 'character'">CHARACTER</xsl:when>
				<xsl:when test="@type = 'string'">STRING</xsl:when>
				<xsl:when test="@type = 'natural'">NATURAL</xsl:when>
				<xsl:when test="@type = 'boolean'">BOOLEAN</xsl:when>
				<xsl:when test="@type = 'variant'">VARIANT</xsl:when>
			</xsl:choose>
		</li>
	</xsl:template>

	<xsl:template match="file">
		<li>
			<xsl:value-of select="@name" /> IS FILE OF <xsl:value-of select="@of-type" />
		</li>
	</xsl:template>
	
	<xsl:template match="array">
		<li>
			<xsl:value-of select="@name" /> IS ARRAY 
			<xsl:choose>
				<xsl:when test="@length">
					WITH LENGTH <xsl:value-of select="@length" />
				</xsl:when>
				<xsl:otherwise>
					FROM  <xsl:value-of select="@min" />
					TO  <xsl:value-of select="@max" />
				</xsl:otherwise>
			</xsl:choose>
			OF <xsl:value-of select="@of-type" />
		</li>
	</xsl:template>
	
	<xsl:template match="list">
		<li>
			<xsl:value-of select="@name" /> IS LIST OF <xsl:value-of select="@of-type" />
		</li>
	</xsl:template>
	
	<xsl:template match="queue">
		<li>
			<xsl:value-of select="@name" /> IS QUEUE OF <xsl:value-of select="@of-type" />
		</li>
	</xsl:template>
	
	<xsl:template match="stack">
		<li>
			<xsl:value-of select="@name" /> IS STACK OF <xsl:value-of select="@of-type" />
		</li>
	</xsl:template>
	
	<xsl:template match="set">
		<li>
			<xsl:value-of select="@name" /> IS SET OF <xsl:value-of select="@of-type" />
		</li>
	</xsl:template>
	
	<xsl:template match="hashtable">
		<li>
			<xsl:value-of select="@name" /> IS HASHTABLE WITH <xsl:value-of select="@type-of-key" /> AS TYPE OF KEY 
			AND <xsl:value-of select="@type-of-value" /> AS TYPE OF VALUE
		</li>
	</xsl:template>
	
	<xsl:template match="interval">
		<li>
			<xsl:value-of select="@name" /> IS INTERVAL 
			FROM  <xsl:value-of select="@min" />
			TO  <xsl:value-of select="@max" />
		</li>
	</xsl:template>

	<xsl:template match="enumeration">
		<li>
			<xsl:value-of select="@name" /> IS ENUMERATION 
			(
				<xsl:for-each select="constant">
					<xsl:value-of select="@name" />
					<xsl:if test="@value">
						(<xsl:value-of select="@value" />)
					</xsl:if>
					<xsl:if test="not(position() = last())"> , </xsl:if>
				</xsl:for-each>
			) 
		</li>
	</xsl:template>
	
	<xsl:template match="record">
		<li><xsl:value-of select="@name" /> IS  RECORD
			<ul style="list-style-type: none;padding-left: 1em;">
				<xsl:apply-templates />
			</ul>
		</li>
		<li>END-RECORD</li>
	</xsl:template>
	
</xsl:stylesheet>
