<?xml version="1.0" encoding="UTF-8"?>
<!-- Copyright (C) 2012 Oleh Radvanskyj -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions">
	<xsl:output method="html" encoding="UTF-8" />
	
	<!-- functions -->
	
	<xsl:template match="solution | package" mode="table_of_contents_functions">
		<xsl:if test="functions">
			<li><a href="#{generate-id(functions)}">Functions</a>
				<ul>
					<xsl:for-each select="functions/function">
						<li>
							<xsl:choose>
								<xsl:when test="@id"><a href="#{@id}">
									<xsl:apply-templates select="." mode="table_of_contents_function_header" />
								</a></xsl:when>
								<xsl:otherwise><a href="#{generate-id(.)}-header">
									<xsl:apply-templates select="." mode="table_of_contents_function_header" />
								</a></xsl:otherwise>
							</xsl:choose>
						</li>
					</xsl:for-each>
				</ul>
			</li>
		</xsl:if>		
	</xsl:template>
	
	<xsl:template match="function" mode="table_of_contents_function_header">
		<xsl:value-of select="@name" />
		(
		<xsl:if test="parameters">
			<ul class="parameters" style="padding-left:0">
				<xsl:for-each select="parameters/parameter">
					<li>
						<xsl:value-of select="@name" />
						:
						<xsl:choose>
							<xsl:when test="@primitive">
								<xsl:choose>
									<xsl:when test="@primitive = 'integer'">INTEGER</xsl:when>
									<xsl:when test="@primitive = 'real'">REAL</xsl:when>
									<xsl:when test="@primitive = 'pointer'">POINTER</xsl:when>
									<xsl:when test="@primitive = 'character'">CHARACTER</xsl:when>
									<xsl:when test="@primitive = 'string'">STRING</xsl:when>
									<xsl:when test="@primitive = 'natural'">NATURAL</xsl:when>
									<xsl:when test="@primitive = 'boolean'">BOOLEAN</xsl:when>
									<xsl:when test="@primitive = 'variant'">VARIANT</xsl:when>
								</xsl:choose>
							</xsl:when>
							<xsl:otherwise>
								<xsl:choose>
									<xsl:when test="@class"><a href="#{@class}"><xsl:value-of select="@type" /></a></xsl:when>
									<xsl:otherwise><xsl:value-of select="@type" /></xsl:otherwise>
								</xsl:choose>
							</xsl:otherwise>
						</xsl:choose>
					</li>
				</xsl:for-each>
			</ul>
		</xsl:if>
		)
		<xsl:if test="returns">
			: 
			<xsl:choose>
				<xsl:when test="returns/@primitive">
					<xsl:choose>
						<xsl:when test="returns/@primitive = 'integer'">INTEGER</xsl:when>
						<xsl:when test="returns/@primitive = 'real'">REAL</xsl:when>
						<xsl:when test="returns/@primitive = 'pointer'">POINTER</xsl:when>
						<xsl:when test="returns/@primitive = 'character'">CHARACTER</xsl:when>
						<xsl:when test="returns/@primitive = 'string'">STRING</xsl:when>
						<xsl:when test="returns/@primitive = 'natural'">NATURAL</xsl:when>
						<xsl:when test="returns/@primitive = 'boolean'">BOOLEAN</xsl:when>
						<xsl:when test="returns/@primitive = 'variant'">VARIANT</xsl:when>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:choose>
						<xsl:when test="returns/@class"><a href="#{returns/@class}"><xsl:value-of select="returns/@type" /></a></xsl:when>
						<xsl:otherwise><xsl:value-of select="returns/@type" /></xsl:otherwise>
					</xsl:choose> 
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>
	</xsl:template>

	<xsl:template match="solution | package" mode="table_of_contents_test_procedures">
		<xsl:if test="test-procedures">
			<li><a href="#{generate-id(test-procedures)}">Test procedures</a>
				<ul>
					<xsl:for-each select="test-procedures/test-procedure">
						<li>
							<xsl:choose>
								<xsl:when test="@id"><a href="#{@id}"><xsl:value-of select="@name" /></a></xsl:when>
								<xsl:otherwise><a href="#{generate-id(.)}-header"><xsl:value-of select="@name" /></a></xsl:otherwise>
							</xsl:choose>
						</li>
					</xsl:for-each>
				</ul>
			</li>
		</xsl:if>	
	</xsl:template>
	
	 <xsl:template match="solution/functions">
		 <h2 id="{generate-id(.)}">Functions</h2>
		 <xsl:apply-templates select="function" />
	 </xsl:template>
  
	<xsl:template match="solution/functions/function">
		<xsl:choose>
			<xsl:when test="@id"><h3 id="{@id}" onclick="toggleTag('{generate-id(.)}')">
				<xsl:apply-templates select="." mode="function_header" />
			</h3></xsl:when>
			<xsl:otherwise><h3 id="{generate-id(.)}-header" onclick="toggleTag('{generate-id(.)}')">
				<xsl:apply-templates select="." mode="function_header" />
			</h3></xsl:otherwise>
		</xsl:choose>
		<div id="{generate-id(.)}" style="display:none" class="function">
			<xsl:apply-templates select="description" />
			<xsl:apply-templates select="preconditions" />
			<xsl:apply-templates select="postconditions" />
			<xsl:apply-templates select="parameters" />
			<xsl:apply-templates select="returns" />
			<xsl:apply-templates select="exceptions" />
			<xsl:apply-templates select="classes" />
			<xsl:apply-templates select="types" />
			<xsl:apply-templates select="constants" />
			<xsl:apply-templates select="shared-data" />
			<xsl:apply-templates select="data" />
			<xsl:apply-templates select="functions/function" />
			<xsl:apply-templates select="aspects" />
			<xsl:apply-templates select="sequence" />
			<xsl:apply-templates select="algorithm" />
			<xsl:apply-templates select="diagrams" />
			<xsl:apply-templates select="test-procedures/test-procedure" />
		</div>
	</xsl:template>  
	  	
	<xsl:template match="solution/test-procedures">
		<h2 id="{generate-id(.)}">Test procedures</h2>
		<xsl:apply-templates select="test-procedure" />
	</xsl:template>
	
	<xsl:template match="solution/test-procedures/test-procedure">
		<xsl:choose>
			<xsl:when test="@id"><h3 id="{@id}" onclick="toggleTag('{generate-id(.)}')">Test procedure : <xsl:value-of select="@name" /></h3></xsl:when>
			<xsl:otherwise><h3 id="{generate-id(.)}-header" onclick="toggleTag('{generate-id(.)}')">Test procedure : <xsl:value-of select="@name" /></h3></xsl:otherwise>
		</xsl:choose>
		<div id="{generate-id(.)}" style="display:none" class="function">
			<xsl:apply-templates select="description" />
			<xsl:apply-templates select="preconditions" />
			<xsl:apply-templates select="postconditions" />
			<xsl:apply-templates select="parameters" />	
			<xsl:apply-templates select="exceptions" />
			<xsl:apply-templates select="classes" />
			<xsl:apply-templates select="types" />
			<xsl:apply-templates select="constants" />
			<xsl:apply-templates select="shared-data" />
			<xsl:apply-templates select="data" />
			<xsl:apply-templates select="functions/function" />
			<xsl:apply-templates select="aspects" />
			<xsl:apply-templates select="sequence" />
			<xsl:apply-templates select="algorithm" />
			<xsl:apply-templates select="diagrams" />
			<xsl:apply-templates select="test-procedures/test-procedure" />
		</div>
	</xsl:template>

	<!-- level of package -->
	
	<xsl:template match="package/functions">
		<h4 id="{generate-id(.)}">Functions</h4>
		<div class="level_of_model">
			<xsl:apply-templates select="function" />
		</div>
	</xsl:template>
	
	<xsl:template match="package/test-procedures">
		<h4 id="{generate-id(.)}">Test procedures</h4>
		<div class="level_of_model">
			<xsl:apply-templates select="test-procedure" />
		</div>
	</xsl:template>

	<xsl:template match="package/functions/function">
		<xsl:apply-templates select="." mode="inner_function" />
	</xsl:template>
	
	<xsl:template match="package/test-procedures/test-procedure">
		<xsl:apply-templates select="." mode="inner_test_procedure" />
	</xsl:template>

</xsl:stylesheet>
